        TITLE   Book psrlX86.asm - Packed Shift Right Logic (X86)
        PAGE    53,160

;
; psrlX86.asm - Packed Shift Right Logical
;
; This document contains file that you may use at your own risk in your own programs.
;
; This module is designed to accomodate X86 processors in the book.
;
;       NOTE: Needs masm 6.14 or later!
;
;       Created - 19 Dec 2001 - J.Leiterman
;
;       Modifications
;
;               
;
;       Tabs = 8



        .XLIST

        .387
        .686P
        .MMX
        .model flat, C

        include x86.inc
        include KariType.inc
 
        .LIST



        .data

externdef       llMaskBQ:qword



psrlX86  SEGMENT USE32 PUBLIC 'CODE'



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Right Logical 16x8bit by n:{0...7}
;
;void vmp_psrlB( uint8 * const pbD, const uint8 * const pbA, uint count )
;{
;       uint32 msk;
;
;       count &= (8-1);                         // Clip count to data bit size -1
;
;       msk = llMaskBD[count];
;       *(((uint32*)pbD)+0) = (*(((uint32*)pbA)+0) & msk) >> count;
;       *(((uint32*)pbD)+1) = (*(((uint32*)pbA)+1) & msk) >> count;
;       *(((uint32*)pbD)+2) = (*(((uint32*)pbA)+2) & msk) >> count;
;       *(((uint32*)pbD)+3) = (*(((uint32*)pbA)+3) & msk) >> count;
;}                                                                      
;       NOTE: As X86 packed shifting only supports 16/32/64 bits we have to mask for 8bit!
;
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrlBAsmMMX
vmp_psrlBAsmMMX       proc    near pbD:ptr, pbA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pbD)
ASSERT_PTR(pbA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pbD
        mov     eax,pbA
        and     ecx,8-1                         ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        pand    mm0,llMaskBQ[ecx*8]             ; Strip lower bits
        pand    mm1,llMaskBQ[ecx*8]

        psrlq   mm0,mm2                         ; val >> count
        psrlq   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psrlBAsmMMX endp



;_________________________________________________________________________________________________
;       Packed Shift Right Logical 16x8bit by 1
;
;void vmp_psrl1B( uint8 * const pbD, const uint8 * const pbA )
;{
;       *(((uint32*)pbD)+0) = (*(((uint32*)pbA)+0) & 0x7f7f7f7f) >> 1;
;       *(((uint32*)pbD)+1) = (*(((uint32*)pbA)+1) & 0x7f7f7f7f) >> 1;
;       *(((uint32*)pbD)+2) = (*(((uint32*)pbA)+2) & 0x7f7f7f7f) >> 1;
;       *(((uint32*)pbD)+3) = (*(((uint32*)pbA)+3) & 0x7f7f7f7f) >> 1;
;}
;       NOTE: As X86 packed shifting only supports 16/32/64 bits we have to mask for 8bit!
;
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrl1BAsmMMX
vmp_psrl1BAsmMMX       proc    near pbD:ptr, pbA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pbD)
ASSERT_PTR(pbA)

        mov     edx,pbD
        mov     eax,pbA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits

        pand    mm0,llMaskBQ[1*8]                       ; Strip lower bit
        pand    mm1,llMaskBQ[1*8]

        psrlq   mm0,1                           ; val >> 1
        psrlq   mm1,1

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psrl1BAsmMMX        endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Right Logical 8x16bit by n:{0...15}
;
;void vmp_psrlH( uint16 * const phD, const uint16 * const phA, uint count )
;{
;       uint32 msk;
;
;       count &= (16-1);                                // Clip count to data bit size -1
;
;       msk = llMaskHD[count];
;       *(((uint32*)phD)+0) = (*(((uint32*)phA)+0) & msk) >> count;
;       *(((uint32*)phD)+1) = (*(((uint32*)phA)+1) & msk) >> count;
;       *(((uint32*)phD)+2) = (*(((uint32*)phA)+2) & msk) >> count;
;       *(((uint32*)phD)+3) = (*(((uint32*)phA)+3) & msk) >> count;
;}                                                                      
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrlHAsmMMX
vmp_psrlHAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pwD
        mov     eax,pwA
        and     ecx,16-1                                ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        psrlw   mm0,mm2                         ; val >> count (0...15)
        psrlw   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psrlHAsmMMX endp



;_________________________________________________________________________________________________
;       Packed Shift Right Logical 8x16bit by 1
;
;void vmp_psrl1H( uint16 * const phD, const uint16 * const phA )
;{
;       *(((uint32*)phD)+0) = (*(((uint32*)phA)+0) & 0xfffefffe) >> 1;
;       *(((uint32*)phD)+1) = (*(((uint32*)phA)+1) & 0xfffefffe) >> 1;
;       *(((uint32*)phD)+2) = (*(((uint32*)phA)+2) & 0xfffefffe) >> 1;
;       *(((uint32*)phD)+3) = (*(((uint32*)phA)+3) & 0xfffefffe) >> 1;
;}
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrl1HAsmMMX
vmp_psrl1HAsmMMX       proc    near phD:ptr, phA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(phD)
ASSERT_PTR(phA)

        mov     edx,phD
        mov     eax,phA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits

        psrlw   mm0,1                           ; val >> 1
        psrlw   mm1,1

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psrl1HAsmMMX        endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Right Logical 4x32bit by n:{0...31}
;
;void vmp_psrlWGeneric( uint32 * const pwD, const uint32 * const pwA, uint count )
;{
;       count &= (32-1);                                // Clip count to data bit size -1
;
;       *(pwD+0) = (*(pwA+0)) >> count;
;       *(pwD+1) = (*(pwA+1)) >> count;
;       *(pwD+2) = (*(pwA+2)) >> count;
;       *(pwD+3) = (*(pwA+3)) >> count;
;}                                                                      
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrlWAsmMMX
vmp_psrlWAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pwD
        mov     eax,pwA
        and     ecx,32-1                                ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        psrld   mm0,mm2                         ; val >> count (0...15)
        psrld   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psrlWAsmMMX endp



;_________________________________________________________________________________________________
;       Packed Shift Right Logical 4x32bit by 1
;
;void vmp_psrl1W( uint32 * const pwD, const uint32 * const pwA )
;{
;       *(pwD+0) = *(pwA+0) >> 1;
;       *(pwD+1) = *(pwA+1) >> 1;
;       *(pwD+2) = *(pwA+2) >> 1;
;       *(pwD+3) = *(pwA+3) >> 1;
;}
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psrl1WAsmMMX
vmp_psrl1WAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     edx,pwD
        mov     eax,pwA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits

        psrld   mm0,1                           ; val >> 1
        psrld   mm1,1

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psrl1WAsmMMX        endp



psrlX86  ends
        end




