
//
//  PasEx.h - Packed Addition/Subtraction
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      This module is designed for fast 3D math
//
//  Created - 31 Dec, 2001 - James Leiterman
//
//  Modifications
//
//
//  stab = 4
//


#include    "CpuAsm.h"
#include    "KariType.h"                                // Master Definitions


#ifndef __PasEx_H
#define __PasEx_H

#ifdef __cplusplus
extern "C" {
#endif


void PasSetup( const CpuInfo * const pcinfo );

    //  Addition (Un)signed

void vmp_paddBGeneric(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddBAsmMMX(   uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddBAsmSSE2(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddBAsmSSE20( uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );

void vmp_paddHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddHAsmMMX(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );

void vmp_paddWGeneric(  uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_paddWAsmMMX(   uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_paddWAsmSSE2(  uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_paddWAsmSSE20( uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );

    //  Subtraction (Un)signed

void vmp_psubBGeneric(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubBAsmMMX(   uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubBAsmSSE2(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubBAsmSSE20( uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );

void vmp_psubHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubHAsmMMX(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );

void vmp_psubWGeneric(  uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_psubWAsmMMX(   uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_psubWAsmSSE2(  uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
void vmp_psubWAsmSSE20( uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );

    //  Addition signed Saturation

void vmp_paddsBGeneric(  int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_paddsBAsmMMX(   int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_paddsBAsmSSE2(  int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_paddsBAsmSSE20( int8  * const pbD, const int8  * const pbA, const int8  * const pbB );

void vmp_paddsHGeneric(  int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_paddsHAsmMMX(   int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_paddsHAsmSSE2(  int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_paddsHAsmSSE20( int16 * const phD, const int16 * const phA, const int16 * const phB );

    //  Subtraction signed Saturation

void vmp_psubsBGeneric(  int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_psubsBAsmMMX(   int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_psubsBAsmSSE2(  int8  * const pbD, const int8  * const pbA, const int8  * const pbB );
void vmp_psubsBAsmSSE20( int8  * const pbD, const int8  * const pbA, const int8  * const pbB );

void vmp_psubsHGeneric(  int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_psubsHAsmMMX(   int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_psubsHAsmSSE2(  int16 * const phD, const int16 * const phA, const int16 * const phB );
void vmp_psubsHAsmSSE20( int16 * const phD, const int16 * const phA, const int16 * const phB );

    //  Addition Unsigned Saturation

void vmp_paddusBGeneric(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddusBAsmMMX(   uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddusBAsmSSE2(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_paddusBAsmSSE20( uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );

void vmp_paddusHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddusHAsmMMX(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddusHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_paddusHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );

    //  Subtraction Unsigned Saturation

void vmp_psubusBGeneric(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubusBAsmMMX(   uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubusBAsmSSE2(  uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_psubusBAsmSSE20( uint8  * const pbD, const uint8  * const pbA, const uint8  * const pbB );

void vmp_psubusHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubusHAsmMMX(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubusHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_psubusHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );



#ifdef __cplusplus
}
#endif

#endif  //__PasEx_H





