
//
//  Vas3DX86.cpp - 3D Vector Addition & Subtraction
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  NOTE: Because these a 3 float Vectors the 4th float has to be handled as 
//      a special case thus slowing the code down. This is the same as an AoS 
//      orientation. Instead a SoA should be used in conjunction with the quad
//      set of functions.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "Vas3D.h"                                   // 3D Math
#include    "Vas3DEx.h"



//
//  Setup Vec3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void Vas3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);

    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_VecAdd =            vmp_VecAddAsmSSE;
        vmp_VecAdd0 =           vmp_VecAddAsmSSE0;
        vmp_VecSub =            vmp_VecSubAsmSSE;
        vmp_VecSub0 =           vmp_VecSubAsmSSE0;
        vmp_VecAddScalar =      vmp_VecAddScalarAsmSSE;
        vmp_VecAddScalar0 =     vmp_VecAddScalarAsmSSE0;
        vmp_VecSubScalar =      vmp_VecSubScalarAsmSSE;
        vmp_VecSubScalar0 =     vmp_VecSubScalarAsmSSE0;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_VecAdd =            vmp_VecAddAsm3DNow;
        vmp_VecAdd0 =           vmp_VecAddAsm3DNow;
        vmp_VecSub =            vmp_VecSubAsm3DNow;
        vmp_VecSub0 =           vmp_VecSubAsm3DNow;
        vmp_VecAddScalar =      vmp_VecAddScalarAsm3DNow;
        vmp_VecAddScalar0 =     vmp_VecAddScalarAsm3DNow;
        vmp_VecSubScalar =      vmp_VecSubScalarAsm3DNow;
        vmp_VecSubScalar0 =     vmp_VecSubScalarAsm3DNow;
      }
    
    else
      {
        vmp_VecAdd =            vmp_VecAddGeneric;
        vmp_VecAdd0 =           vmp_VecAddGeneric;
        vmp_VecSub =            vmp_VecSubGeneric;
        vmp_VecSub0 =           vmp_VecSubGeneric;
        vmp_VecAddScalar =      vmp_VecAddScalarGeneric;
        vmp_VecAddScalar0 =     vmp_VecAddScalarGeneric;
        vmp_VecSubScalar =      vmp_VecSubScalarGeneric;
        vmp_VecSubScalar0 =     vmp_VecSubScalarGeneric;
      }
}



