        TITLE   Vas3DX86.asm - 80(x)86 3D Math
        PAGE    53,160

;  Vas3D.asm - 3D Math Addition & Subtraction
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

        .686
        .XMM

        .model flat, C


        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST



        .data
        align   16

himsk32 DWORD   000000000h,000000000h,000000000h,0FFFFFFFFh
lomsk96 DWORD   0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,000000000h



;------------------------------------------------------------------------------------------------


Vas3DX86        SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------



;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Summation (Addition)
;
;void vmp_VecAdd(vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
;{
;       pvD->x = pvA->x + pvB->x;
;       pvD->y = pvA->y + pvB->y;
;       pvD->z = pvA->z + pvB->z;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

    align   16
    public  vmp_VecAddAsm3DNow
vmp_VecAddAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
    .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vB)
ASSERT_PTR(vD)

    mov     eax,vA              ; Vector A
    mov     ecx,vB              ; Vector B
    mov     edx,vD              ; Vector Destination

    movq    mm0,[eax]                       ;vA.xy   {Ay Ax}
    movq    mm2,[ecx]                       ;vB.xy   {By Bx}
    movd    mm1,(vmp3DVector PTR [eax]).z   ;vA.z0   {0 Az}
    movd    mm3,(vmp3DVector PTR [ecx]).z   ;vB.z0   {0 Bz}

    pfadd   mm0,mm2                         ;       {Ay+By Ax+Bx}
    pfadd   mm1,mm3                         ;       {0+0   Az+Bz}

    movq    [edx],mm0                       ;       {Ay+By Ax+Bx}
    movd    (vmp3DVector PTR [edx]).z,mm1   ;       {0     Az+Bz}

;;;        femms
        ret
vmp_VecAddAsm3DNow endp



;_________________________________________________________________________________________________

; *** SSE version *** non aligned
;
        align   16
        public  vmp_VecAddAsmSSE0
vmp_VecAddAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vD                          ; Vector Destination
        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movups  xmm2,[eax]
        movups  xmm0,[edx]                      ;vB.xyz# {# Bz By Bx}
        movups  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        andps   xmm2,OWORD PTR himsk32          ;        {# 0  0  0}
        addps   xmm0,xmm1                       ;        {#+# Az+Bz Ay+By Ax+Bx}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az+Bz Ay+By Ax+Bx}

        orps    xmm0,xmm2                       ;        {# Az+Bz Ay+By Ax+Bx}

        movups  [eax],xmm0                      ;        {# Az+Bz Ay+By Ax+Bx}  
        
;;;        emms
        ret
vmp_VecAddAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_VecAddAsmSSE
vmp_VecAddAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)

        mov     eax,vD                          ; Vector Destination
        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movaps  xmm2,[eax]
        movaps  xmm0,[edx]                      ;vB.xyz# {# Bz By Bx}
        movaps  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        andps   xmm2,OWORD PTR himsk32          ;        {# 0  0  0}
        addps   xmm0,xmm1                       ;        {#+# Az+Bz Ay+By Ax+Bx}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az+Bz Ay+By Ax+Bx}

        orps    xmm0,xmm2                       ;        {# Az+Bz Ay+By Ax+Bx}

        movaps  [eax],xmm0                      ;        {# Az+Bz Ay+By Ax+Bx}  
        
;;;        emms
        ret
vmp_VecAddAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Scalar Summation (Addition)
;
;void vmp_VecAddScalar(vmp3DVector * const pvD, const vmp3DVector * const pvA, float fScalar)
;{
;       pvD->x = pvA->x + fScalar;
;       pvD->y = pvA->y + fScalar;
;       pvD->z = pvA->z + fScalar;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecAddScalarAsm3DNow
vmp_VecAddScalarAsm3DNow       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        movd    mm2,fScalar                     ; fScalar {0 s}
        mov     ecx,vA                          ; Vector A
        movq    mm3,mm2                         ; fScalar {0 s}
        punpckldq mm2,mm2                               ;         {s s}
        
        mov     eax,vD                          ; Vector Destination

        movq    mm0,[ecx]                       ;vA.xy   {Ay Ax}
        movd    mm1,(vmp3DVector PTR [ecx]).z   ;vA.z0   {0  Az}

        pfadd   mm0,mm2                         ;       {Ay+s Ax+s}
        pfadd   mm1,mm3                         ;       {0+0  Az+s}

        movq    [eax],mm0                       ;       {Ay+By Ax+Bx}
        movd    (vmp3DVector PTR [eax]).z,mm1   ;       {0     Az+Bz}
        
;;;        femms
        ret
vmp_VecAddScalarAsm3DNow endp


;_________________________________________________________________________________________________

; *** SSE version *** non aligned
;
        align   16
        public  vmp_VecAddScalarAsmSSE0
vmp_VecAddScalarAsmSSE0       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        pxor    xmm1,xmm1                       ;        (0 0 0 0)
        mov     eax,vD                          ; Vector Destination
        mov     ecx,vA                          ; Vector A

        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movups  xmm0,[ecx]                      ;vA.xyz# {? Az Ay Ax}
        movups  xmm2,[eax]                      ;        {#  #  # #}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az Ay Ax}
        shufps  xmm1,xmm1,0c0h          ; 3 0 0 0         {0  s  s  s}
        andps   xmm2,OWORD PTR himsk32          ;        {#  0  0  0}

        addps   xmm0,xmm1                       ;        {0+0 Az+s Ay+s Ax+s}

        orps    xmm0,xmm2                       ;        { #  Az+s Ay+s Ax+s}

        movups  [eax],xmm0                      ;        { #  Az+s Ay+s Ax+s}   
        
;;;        emms
        ret
vmp_VecAddScalarAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_VecAddScalarAsmSSE
vmp_VecAddScalarAsmSSE       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)

        pxor    xmm1,xmm1                       ;        (0 0 0 0)
        mov     eax,vD                          ; Vector Destination
        mov     ecx,vA                          ; Vector A

        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movaps  xmm0,[ecx]                      ;vA.xyz# {? Az Ay Ax}
        movaps  xmm2,[eax]                      ;        {#  #  # #}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az Ay Ax}
        shufps  xmm1,xmm1,0c0h          ; 3 0 0 0         {0  s  s  s}
        andps   xmm2,OWORD PTR himsk32          ;        {#  0  0  0}

        addps   xmm0,xmm1                       ;        {0 Az+s Ay+s Ax+s}

        orps    xmm0,xmm2                       ;        {# Az+s Ay+s Ax+s}

        movaps  [eax],xmm0                      ;        {# Az+s Ay+s Ax+s}     
        
;;;        emms
        ret
vmp_VecAddScalarAsmSSE endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Difference (Subtraction)
;
;void vmp_VecSub(vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
;{
;       pvD->x = pvA->x - pvB->x;
;       pvD->y = pvA->y - pvB->y;
;       pvD->z = pvA->z - pvB->z;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecSubAsm3DNow
vmp_VecSubAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        movq    mm0,[ecx]                       ;vA.xy   {Ay Ax}
        movq    mm2,[edx]                       ;vB.xy   {By Bx}
        movd    mm1,(vmp3DVector PTR [ecx]).z   ;vA.z0   {0 Az}
        movd    mm3,(vmp3DVector PTR [edx]).z   ;vB.z0   {0 Bz}

        pfsub   mm0,mm2                         ;       {Ay-By Ax-Bx}
        pfsub   mm1,mm3                         ;       {0-0  Az-Bz}

        movq    [eax],mm0                       ;       {Ay-By Ax-Bx}
        movd    (vmp3DVector PTR [eax]).z,mm1   ;       {0     Az-Bz}
        
;;;        femms
        ret
vmp_VecSubAsm3DNow endp


;_________________________________________________________________________________________________

; *** SSE version *** non aligned
;
        align   16
        public  vmp_VecSubAsmSSE0
vmp_VecSubAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vD                          ; Vector Destination
        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movups  xmm2,[eax]
        movups  xmm1,[edx]                      ;vB.xyz# {# Bz By Bx}
        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        andps   xmm2,OWORD PTR himsk32          ;        {# 0  0  0}
        subps   xmm0,xmm1                       ;        {#-# Az-Bz Ay-By Ax-Bx}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az-Bz Ay-By Ax-Bx}

        orps    xmm0,xmm2                       ;        {# Az-Bz Ay-By Ax-Bx}

        movups  [eax],xmm0                      ;        {# Az-Bz Ay-By Ax-Bx}  
        
;;;        emms
        ret
vmp_VecSubAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_VecSubAsmSSE
vmp_VecSubAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)

        mov     eax,vD                          ; Vector Destination
        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movaps  xmm2,[eax]
        movaps  xmm1,[edx]                      ;vB.xyz# {# Bz By Bx}
        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        andps   xmm2,OWORD PTR himsk32          ;        {# 0  0  0}
        subps   xmm0,xmm1                       ;        {#-# Az-Bz Ay-By Ax-Bx}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az-Bz Ay-By Ax-Bx}

        orps    xmm0,xmm2                       ;        {# Az-Bz Ay-By Ax-Bx}

        movaps  [eax],xmm0                      ;        {# Az-Bz Ay-By Ax-Bx}  
        
;;;        emms
        ret
vmp_VecSubAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Scalar Difference (Subtraction)
;
;void vmp_VecSubScalar(vmp3DVector * const pvD, const vmp3DVector * const pvA, float fScalar)
;{
;       pvD->x = pvA->x - fScalar;
;       pvD->y = pvA->y - fScalar;
;       pvD->z = pvA->z - fScalar;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecSubScalarAsm3DNow
vmp_VecSubScalarAsm3DNow       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        movd    mm2,fScalar                     ; fScalar {0 s}
        mov     ecx,vA                          ; Vector A
        movq    mm3,mm2                         ; fScalar {0 s}
        punpckldq mm2,mm2                               ;         {s s}
        
        mov     eax,vD                          ; Vector Destination

        movq    mm0,[ecx]                       ;vA.xy   {Ay Ax}
        movd    mm1,(vmp3DVector PTR [ecx]).z   ;vA.z0   {0  Az}

        pfsub   mm0,mm2                         ;       {Ay-s Ax-s}
        pfsub   mm1,mm3                         ;       {0-0  Az-s}

        movq    [eax],mm0                       ;       {Ay-s Ax-s}
        movd    (vmp3DVector PTR [eax]).z,mm1   ;       {0    Az-s}
        
;;;        femms
        ret
vmp_VecSubScalarAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_VecSubScalarAsmSSE0
vmp_VecSubScalarAsmSSE0       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        pxor    xmm1,xmm1                       ;        (0 0 0 0)
        mov     eax,vD                          ; Vector Destination
        mov     ecx,vA                          ; Vector A

        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm2,[eax]                      ;        {#  #  # #}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az Ay Ax}
        shufps  xmm1,xmm1,0c0h          ; 3 0 0 0         {0  s  s  s}
        andps   xmm2,OWORD PTR himsk32          ;        {# 0  0  0}

        subps   xmm0,xmm1                       ;        {0 Az-s Ay-s Ax-s}

        orps    xmm0,xmm2                       ;        {# Az-s Ay-s Ax-s}

        movups  [eax],xmm0                      ;        {# Az-s Ay-s Ax-s}     
        
;;;        emms
        ret
vmp_VecSubScalarAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_VecSubScalarAsmSSE
vmp_VecSubScalarAsmSSE       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)

        pxor    xmm1,xmm1                       ;        (0 0 0 0)
        mov     eax,vD                          ; Vector Destination
        mov     ecx,vA                          ; Vector A

        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm2,[eax]                      ;        {#  #  #  #}

        andps   xmm0,OWORD PTR lomsk96          ;        {0 Az Ay Ax}
        shufps  xmm1,xmm1,0c0h          ; 3 0 0 0         {0  s  s  s}
        andps   xmm2,OWORD PTR himsk32          ;        {#  0  0  0}

        subps   xmm0,xmm1                       ;        {0 Az-s Ay-s Ax-s}

        orps    xmm0,xmm2                       ;        {# Az-s Ay-s Ax-s}

        movaps  [eax],xmm0                      ;        {# Az-s Ay-s Ax-s}     
        
;;;        emms
        ret
vmp_VecSubScalarAsmSSE endp



Vas3DX86        ends
        end

