
//
//  Vmd3DEx.cpp - 3D Vector Multiplication & Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"
#include    "Vmp3D.h"
#include    "Vmp3DEx.h"
#include    "Vmd3D.h"                                   // 3D Math
#include    "Vmd3DEx.h"



//
//  Setup Vec3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void Vmd3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);

    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_VecMul =            vmp_VecMulAsmSSE;
        vmp_VecMul0 =           vmp_VecMulAsmSSE0;
        vmp_VecMAdd =           vmp_VecMAddAsmSSE;
        vmp_VecMAdd0 =          vmp_VecMAddAsmSSE0;
        vmp_VecMSub =           vmp_VecMSubAsmSSE;
        vmp_VecMSub0 =          vmp_VecMSubAsmSSE0;
        vmp_VecDiv =            vmp_VecDivAsmSSE;
        vmp_VecDiv0 =           vmp_VecDivAsmSSE0;
        vmp_VecDivFast =        vmp_VecDivFastAsmSSE;
        vmp_VecDivFast0 =       vmp_VecDivFastAsmSSE0;
        vmp_VecScale =          vmp_VecScaleAsmSSE;
        vmp_VecScale0 =         vmp_VecScaleAsmSSE0;
        vmp_DotProduct =        vmp_DotProductAsmSSE;
        vmp_DotProduct0 =       vmp_DotProductAsmSSE0;
        vmp_CrossProduct =      vmp_CrossProductAsmSSE;
        vmp_CrossProduct0 =     vmp_CrossProductAsmSSE0;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_VecMul =            vmp_VecMulAsm3DNow;
        vmp_VecMul0 =           vmp_VecMulAsm3DNow;
        vmp_VecMAdd =           vmp_VecMAddAsm3DNow;
        vmp_VecMAdd0 =          vmp_VecMAddAsm3DNow;
        vmp_VecMSub =           vmp_VecMSubAsm3DNow;
        vmp_VecMSub0 =          vmp_VecMSubAsm3DNow;
        vmp_VecDiv =            vmp_VecDivAsm3DNow;
        vmp_VecDiv0 =           vmp_VecDivAsm3DNow;
        vmp_VecDivFast =        vmp_VecDivFastAsm3DNow; //***FAST***
        vmp_VecDivFast0 =       vmp_VecDivFastAsm3DNow; //***FAST***
        vmp_VecScale =          vmp_VecScaleAsm3DNow;
        vmp_VecScale0 =         vmp_VecScaleAsm3DNow;
        vmp_DotProduct =        vmp_DotProductAsm3DNow;
        vmp_DotProduct0 =       vmp_DotProductAsm3DNow;
        vmp_CrossProduct =      vmp_CrossProductAsm3DNow;
        vmp_CrossProduct0 =     vmp_CrossProductAsm3DNow;
      }
    
    else

      {
        vmp_VecMul =            vmp_VecMulGeneric;
        vmp_VecMul0 =           vmp_VecMulGeneric;
        vmp_VecMAdd =           vmp_VecMAddGeneric;
        vmp_VecMAdd0 =          vmp_VecMAddGeneric;
        vmp_VecMSub =           vmp_VecMSubGeneric;
        vmp_VecMSub0 =          vmp_VecMSubGeneric;
        vmp_VecDiv =            vmp_VecDivGeneric;
        vmp_VecDiv0 =           vmp_VecDivGeneric;
        vmp_VecDivFast =        vmp_VecDivGeneric;
        vmp_VecDivFast0 =       vmp_VecDivGeneric;
        vmp_VecScale =          vmp_VecScaleGeneric;
        vmp_VecScale0 =         vmp_VecScaleGeneric;
        vmp_DotProduct =        vmp_DotProductGeneric;
        vmp_DotProduct0 =       vmp_DotProductGeneric;
        vmp_CrossProduct =      vmp_CrossProductGeneric;
        vmp_CrossProduct0 =     vmp_CrossProductGeneric;
      }
}


