        TITLE   QVsf3DX86.asm - 80(x)86 3D (Quad) Math Special Function
        PAGE    53,160

;  QVsf3D.asm - 3D (Quad) Vector Special Function Math
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8


        .XLIST

;;;        .387
;;;     .586P   .586    .686   .686P
;;;      .K3D    .XMM    .MMX

        .686
        .XMM

        .model flat, C

;ifdef _DEBUG
USE_ASSERT      =       1
;endif


        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST


        .data
        align   16

ONES    REAL4   1.0,1.0,1.0,1.0
ONEHIGH REAL4   0.0,0.0,0.0,1.0

vTiny   REAL4   0.0000001,0.0000001,0.0000001,0.0000001
vZero   REAL4   0.0,0.0,0.0,0.0

;lomsk96        OWORD   000000000FFFFFFFFFFFFFFFFFFFFFFFFh      ; 00000000 FFFFFFFF FFFFFFFF FFFFFFFF
lomsk96 DWORD   0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,000000000h

tst REAL4 4.0,3.0,2.0,1.0
foo REAL4 0.0,0.0,0.0,0.0
foo2 REAL4 0.0,0.0,0.0,0.0


;------------------------------------------------------------------------------------------------


QVsf3DX86       SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Minimum
;
;
;void vmp_QVecMin( vmp3DQVector * const pvD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB )
;{
;       pvD->x = (pvA->x < pvB->x) ? pvA->x : pvB->x;
;       pvD->y = (pvA->y < pvB->y) ? pvA->y : pvB->y;
;       pvD->z = (pvA->z < pvB->z) ? pvA->z : pvB->z;
;       pvD->w = (pvA->w < pvB->w) ? pvA->w : pvB->w;
;}

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecMinAsm3DNow
vmp_QVecMinAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movq    mm0,[eax+0]                     ;vA.xy  {Ay   Ax}
        movq    mm1,[eax+8]                     ;vA.zw  {Aw   Az}
        movq    mm2,[ecx+0]                     ;vB.xy  {By   Bx}
        movq    mm3,[ecx+8]                     ;vB.zw  {Bw   Bz}

        pfmin   mm0,mm2                         ;    {Ay<By Ax<Bx}
        pfmin   mm1,mm3                         ;    {Aw<Bw Az<Bz}
        
        movq    [edx+0],mm0                     ;    {Miny Minx}
        movq    [edx+8],mm1                     ;    {Minw Minz}
;;;        femms
        ret
vmp_QVecMinAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecMinAsmSSE0
vmp_QVecMinAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movups  xmm0,[eax]                      ;vA.xyzw {Aw Az Ay Ax}
        movups  xmm1,[ecx]                      ;vB.xyzw {Bw Bz By Bx}

        minps   xmm0,xmm1       

        movups  [edx],xmm0                      ; Save Min Vals
;;;        emms
        ret
vmp_QVecMinAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecMinAsmSSE
vmp_QVecMinAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movaps  xmm0,[eax]                      ;vA.xyzw {Aw Az Ay Ax}
        movaps  xmm1,[ecx]                      ;vB.xyzw {Bw Bz By Bx}

        minps   xmm0,xmm1       

        movaps  [edx],xmm0                      ; Save Min vals
;;;        emms
        ret
vmp_QVecMinAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Maximum
;
;
;void vmp_QVecMax( vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB )
;{
;       pvD->x = (pvA->x > pvB->x) ? pvA->x : pvB->x;
;       pvD->y = (pvA->y > pvB->y) ? pvA->y : pvB->y;
;       pvD->z = (pvA->z > pvB->z) ? pvA->z : pvB->z;
;       pvD->w = (pvA->w > pvB->w) ? pvA->w : pvB->w;
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecMaxAsm3DNow
vmp_QVecMaxAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movq    mm0,[eax+0]                     ;vA.xy  {Ay   Ax}
        movq    mm1,[eax+8]                     ;vA.zw  {Aw   Az}
        movq    mm2,[ecx+0]                     ;vB.xy  {By   Bx}
        movq    mm3,[ecx+8]                     ;vB.zw  {Bw   Bz}

        pfmax   mm0,mm2                         ;    {Ay>By Ax>Bx}
        pfmax   mm1,mm3                         ;    {Aw>Bw Az>Bz}
        
        movq    [edx+0],mm0                     ;    {Miny Minx}
        movq    [edx+8],mm1                     ;    {Minw Minz}
;;;        femms
        ret
vmp_QVecMaxAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecMaxAsmSSE0
vmp_QVecMaxAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movups  xmm0,[eax]                      ;vA.xyzw {Aw Az Ay Ax}
        movups  xmm1,[ecx]                      ;vB.xyzw {Bw Bz By Bx}

        maxps   xmm0,xmm1       

        movups  [edx],xmm0                      ; Save Max Vals
;;;        emms
        ret
vmp_QVecMaxAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecMaxAsmSSE
vmp_QVecMaxAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movaps  xmm0,[eax]                      ;vA.xyzw {Aw Az Ay Ax}
        movaps  xmm1,[ecx]                      ;vB.xyzw {Bw Bz By Bx}

        maxps   xmm0,xmm1       

        movaps  [edx],xmm0                      ; Save Max Vals
;;;        emms
        ret
vmp_QVecMaxAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Square Root
;       (Similar to VecMagnitude)
;
;void vmp_QVecSqrt(vmp3DQVector * const pvD, const vmp3DQVector * const pvA)
;{
;       pvD->x = sqrtf( pvA->x );
;       pvD->y = sqrtf( pvA->y );
;       pvD->z = sqrtf( pvA->z );
;       pvD->w = sqrtf( pvA->w );
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***     ***FAST***  15bit Precision
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecSqrtFastAsm3DNow
vmp_QVecSqrtFastAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vD)
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movq    mm0,[ecx+0]                     ;vA.xy  {Ay   Ax}
        movd    mm6,(vmp3DQVector PTR [ecx]).y  ;       {0    Ay}
        movq    mm1,[ecx+8]                     ;vA.zw  {Aw   Az}
        movd    mm5,(vmp3DQVector PTR [ecx]).w  ;       {0    Aw}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm3,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Ax)  1/sqrt(Ax)}
        pfrsqrt     mm7,mm6                 ;                   {1/sqrt(Ay)  1/sqrt(Ay)}
        pfrsqrt     mm2,mm1              ;X0=f(z)=1/sqrt(z)  {1/sqrt(Az)  1/sqrt(Az)}
        pfrsqrt     mm4,mm5                 ;                   {1/sqrt(Aw)  1/sqrt(Aw)}

         ;      Calculate 1/sqrt() accurate to 15bits
        punpckldq   mm3,mm7             ;            { 1/sqrt(Ay)  1/sqrt(Ax)}
        punpckldq   mm2,mm4             ;            { 1/sqrt(Aw)  1/sqrt(Az)}
        pfmul       mm0,mm3             ;            {Ay/sqrt(Ay) Ax/sqrt(Ax)}
        pfmul       mm1,mm2             ;            {Aw/sqrt(Aw) Az/sqrt(Az)}
        
        movq    [eax+0],mm0             ;            {sqrt(Ay) sqrt(Ax)}
        movq    [eax+8],mm1             ;            {sqrt(Aw) sqrt(Az)}
;;;        femms
        ret
vmp_QVecSqrtFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecSqrtAsm3DNow
vmp_QVecSqrtAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vD)
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movq    mm0,[ecx+0]                     ;vA.xy  {Ay   Ax}
        movd    mm6,(vmp3DQVector PTR [ecx]).y  ;       {0    Ay}
        movq    mm1,[ecx+8]                     ;vA.zw  {Aw   Az}
        movd    mm5,(vmp3DQVector PTR [ecx]).w  ;       {0    Aw}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm3,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Ax)  1/sqrt(Ax)}
        pfrsqrt     mm7,mm6                 ;                   {1/sqrt(Ay)  1/sqrt(Ay)}
        pfrsqrt     mm2,mm1              ;X0=f(z)=1/sqrt(z)  {1/sqrt(Az)  1/sqrt(Az)}
        pfrsqrt     mm4,mm5                 ;                   {1/sqrt(Aw)  1/sqrt(Aw)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        punpckldq   mm3,mm7             ;            { 1/sqrt(Ay)  1/sqrt(Ax)}
        punpckldq   mm2,mm4             ;            { 1/sqrt(Aw)  1/sqrt(Az)}

        movq        mm6,mm3             ;            {1/sqrt(Ay)  1/sqrt(Ax)}
        movq        mm5,mm2             ;            {1/sqrt(Aw)  1/sqrt(Az)}

        pfmul       mm3,mm3             ;X1=f(x0,x0) {1/Ay        1/Ax}
        pfmul       mm2,mm2             ;X1=f(x0,x0) {1/Aw        1/Az}

        pfrsqit1    mm3,mm0             ;X2=f(x,x1)     {1st step}
        pfrsqit1    mm2,mm1             ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit
        pfrcpit2    mm3,mm6             ; y=f(x2,x0)    {2nd step}
        pfrcpit2    mm2,mm5             ; y=f(x2,x0)    {2nd step}

        pfmul       mm0,mm3             ;            {Ay/sqrt(Ay) Ax/sqrt(Ax)}
        pfmul       mm1,mm2             ;            {Aw/sqrt(Aw) Az/sqrt(Az)}
        
        movq    [eax+0],mm0             ;            {sqrt(Ay) sqrt(Ax)}
        movq    [eax+8],mm1             ;            {sqrt(Aw) sqrt(Az)}
;;;        femms
        ret
vmp_QVecSqrtAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecSqrtAsmSSE0
vmp_QVecSqrtAsmSSE0       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movups  xmm0,[ecx]                      ;vA.xyzw { Aw  Az  Ay  Ax}
        sqrtps  xmm0,xmm0                       ; SqRoot {rAw rAz rAy rAx}
        movups  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_QVecSqrtAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecSqrtAsmSSE
vmp_QVecSqrtAsmSSE       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movaps  xmm0,[ecx]                      ;vA.xyzw { Aw  Az  Ay  Ax}
        sqrtps  xmm0,xmm0                       ; SqRoot {rAw rAz rAy rAx}
        movaps  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_QVecSqrtAsmSSE endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecSqrtFastAsmSSE0
vmp_QVecSqrtFastAsmSSE0       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movups  xmm0,[ecx]                      ;vA.xyzw { Aw  Az  Ay  Ax}
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        movaps  xmm1,xmm0

        cmpps   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpps   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtps xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulps   xmm0,xmm1       ; {Aw*1/sqrt(Aw) ... Ax*1/sqrt(Ax)}

        movups  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_QVecSqrtFastAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecSqrtFastAsmSSE
vmp_QVecSqrtFastAsmSSE       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DQVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).z )
ASSERT_FNEG( (vmp3DQVector PTR [ecx]).w )

        movaps  xmm0,[ecx]                      ;vA.xyzw { Aw  Az  Ay  Ax}
        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0

        cmpps   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpps   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtps xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulps   xmm0,xmm1       ; {Aw*1/sqrt(Aw) ... Ax*1/sqrt(Ax)}

        movaps  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_QVecSqrtFastAsmSSE endp



;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Normalization
;
;
;void vmp_QVecNormalizeFast(vmp3DQVector * const pvD, const vmp3DQVector * const pvA)
;{
;       float fMag;
;       fMag = sqrtf( pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z );
;       if ( fMag < 0.0001f ) { // If too close to zero? Don't want a divide by zero!
;               pvD->x = pvA->x;  // Original values
;               pvD->y = pvA->y;
;               pvD->z = pvA->z;
;       } else {                        // Not zero so 1/n
;               fMag = 1.0f / fMag;
;               pvD->x = pvA->x * fMag;
;               pvD->y = pvA->y * fMag;
;               pvD->z = pvA->z * fMag;
;       }
;       pvD->w = 0.0;
;}
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!
;
;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecNormalizeFastAsm3DNow
vmp_QVecNormalizeFastAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0    Az}
        movq    mm4,mm1                         ;       {Ay   Ax}
        movq    mm3,mm0                         ;       {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy                 fMag= r=AzAz+AyAy+AxAx

        ; too close zero ...???  1.0 / 10000.0 ???

        movd    ecx,mm0                         ; fMag
        cmp     ecx,FLOAT0001                   ; 0.0001f   3f800000h
        jl      short zmag                      ;just set vD=vA!!!

        ; Not too close so calculate...         r=AzAz+AyAy+AxAx

        pfrsqrt mm1,mm0                  ;X0=f(x)=m=1/sqrt(r)  {1/sqrt(r) 1/sqrt(r)}
;       pfmul    mm1,mm1                        ;X1=f(x0,x0)       {1/r  1/r}
        pfmul   mm4,mm1                         ;{Ny Nx}= {Ay/m Ax/m}
        pfmul   mm3,mm1                         ;{ 0 Nz}= { 0/m Az/m}

        ;       Save Resulting {x y z} Normals

zmag:   movq    [eax+0],mm4                     ;{Ny Nx}
        movq    [eax+8],mm3                     ;{0  Nz}
        
;;;        femms
        ret
vmp_QVecNormalizeFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecNormalizeAsm3DNow
vmp_QVecNormalizeAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0    Az}
        movq    mm4,mm1                         ;       {Ay   Ax}
        movq    mm3,mm0                         ;       {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy

        ; too close zero ...???  1.0 / 10000.0 ???

        movd    ecx,mm0
        cmp     ecx,FLOAT0001                   ; 0.0001f
        jl      short zmag                      ;just set vD=vA!!!

        ; Not too close so calculate...         ;r=AzAz+AyAy+AxAx

        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x) {1/sqrt(r) 1/sqrt(r)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        movq        mm2,mm1             ;               {1/sqrt(r) 1/sqrt(r)}
        pfmul       mm1,mm1             ;X1=f(x0,x0)    {1/r 1/r}
        pfrsqit1    mm1,mm0             ;X2=f(x,x1)     {1st step}

         ; *** mm1 = Magnitude ***
         ;      Calculate sqrt() = (1/mag)  24bit
        pfrcpit2    mm1,mm2             ; y=f(x2,x0) {2nd step}  {# m}

        punpckldq mm1,mm1                               ;         {1/m   1/m}

        pfmul   mm4,mm1                         ;{Ny Nx}= {Ay/m Ax/m}
        pfmul   mm3,mm1                         ;{ 0 Nz}= { 0/m Az/m}

        ;       Save Resulting {x y z} Normals

zmag:   movq    [eax+0],mm4                     ;    {Ny Nx}
        movq    [eax+8],mm3                     ;    {0  Nz}
        
;;;        femms
        ret
vmp_QVecNormalizeAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecNormalizeAsmSSE0
vmp_QVecNormalizeAsmSSE0       proc    near , vD:ptr, vA:ptr
        local uflow:REAL4

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        andps   xmm0,lomsk96                    ;        {0 Az Ay Ax}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAx AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???
        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        sqrtps  xmm0,xmm1                       ; SqRoot  {1 rMz rMy rMx}
        divps   xmm6,xmm0                       ; n/m     {0 rNz rNy rNx}

zmag:   andps   xmm6,lomsk96                    ;{0 Nz Ny Nx}
        movups  [eax],xmm6                      ; Save xyz0

;;;        emms
        ret
vmp_QVecNormalizeAsmSSE0 endp


;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecNormalizeAsmSSE
vmp_QVecNormalizeAsmSSE       proc    near , vD:ptr, vA:ptr
        local uflow:DWORD

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        andps   xmm0,lomsk96                    ;        {0 Az Ay Ax}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAz AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???

        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        sqrtps  xmm0,xmm1                       ; SqRoot  {1 rMz rMy rMx}
        divps   xmm6,xmm0                       ; n/m     {0 rNz rNy rNx}

zmag:   andps   xmm6,lomsk96                    ;{0 Nz Ny Nx}
        movaps  [eax],xmm6                      ; Save xyz0

;;;        emms
        ret
vmp_QVecNormalizeAsmSSE endp



;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecNormalizeFastAsmSSE0
vmp_QVecNormalizeFastAsmSSE0       proc    near , vD:ptr, vA:ptr
        local uflow:REAL4

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        andps   xmm0,lomsk96                    ;        {0 Az Ay Ax}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAx AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???
        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        rsqrtps xmm0,xmm1                       ; SqRoot  {1/1     1/Mz      1/My      1/Mx}
        mulps   xmm6,xmm0                       ; n/m     {0   Az*(1/Mz) Ay*(1/My) Ax*(1/Mx)}

zmag:   andps   xmm6,lomsk96                    ;{0 Nz Ny Nx}
        movups  [eax],xmm6                      ; Save xyz0

;;;        emms
        ret
vmp_QVecNormalizeFastAsmSSE0 endp


;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecNormalizeFastAsmSSE
vmp_QVecNormalizeFastAsmSSE       proc    near , vD:ptr, vA:ptr
        local uflow:DWORD

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        andps   xmm0,lomsk96                    ;        {0 Az Ay Ax}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAz AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???

        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        rsqrtps xmm0,xmm1                       ; SqRoot  {1/1     1/Mz      1/My      1/Mx}
        mulps   xmm6,xmm0                       ; n/m     {0   Az*(1/Mz) Ay*(1/My) Ax*(1/Mx)}

zmag:   andps   xmm6,lomsk96                    ;{0 Nz Ny Nx}
        movaps  [eax],xmm6                      ; Save xyz0

;;;        emms
        ret
vmp_QVecNormalizeFastAsmSSE endp




;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Magnitude - (3D Distance from {0,0,0})
;
;
;void vmp_QVecMagnitude(float * const pfD, const vmp3DQVector * const pvA)
;{
;       *pfD = sqrtf( pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z );
;}
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!
;
;_________________________________________________________________________________________________
; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecMagnitudeFastAsm3DNow
vmp_QVecMagnitudeFastAsm3DNow       proc    near , pfD:ptr, pvA:ptr
        .K3D
;;;        femms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}
                                                ;      r=AzAz+AyAy+AxAx
         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(r) 1/sqrt(r)}
        pfmul       mm0,mm1                     ;         {r/sqrt(r) r/sqrt(r)}
                                                ; r = r/sqrt(r)
        movd    [eax],mm0                       ; Save resulting distance float
        
;;;        femms
        ret
vmp_QVecMagnitudeFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecMagnitudeAsm3DNow
vmp_QVecMagnitudeAsm3DNow       proc    near , pfD:ptr, pvA:ptr
        .K3D
;;;        femms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+AxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x)  {#####  1/sqrt(AzAz+AyAy+AxAx)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        movq        mm2,mm1             ;            {#####  1/sqrt(AzAz+AyAy+AxAx)}
        pfmul       mm1,mm1             ;X1=f(x0,x0) {#####  1/(AzAz+AyAy+AxAx)}
        pfrsqit1    mm1,mm0             ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit
        pfrcpit2    mm1,mm2             ; y=f(x2,x0)    {2nd step}
        pfmul       mm0,mm1

        movd    [eax],mm0                       ; Save resulting distance float
        
;;;        femms
        ret
vmp_QVecMagnitudeAsm3DNow endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecMagnitudeAsmSSE0
vmp_QVecMagnitudeAsmSSE0       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mulps   xmm0,xmm0                       ; {AwAw AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwAw AwAw AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {AwAw AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {AwAw AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {AwAw AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        sqrtss  xmm0,xmm2                       ; SqRoot
        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_QVecMagnitudeAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecMagnitudeAsmSSE
vmp_QVecMagnitudeAsmSSE       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mulps   xmm0,xmm0                       ; {AwAw AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwAw AwAw AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {AwAw AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {AwAw AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {AwAw AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        sqrtss  xmm0,xmm2                       ; SqRoot
        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_QVecMagnitudeAsmSSE endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QVecMagnitudeFastAsmSSE0
vmp_QVecMagnitudeFastAsmSSE0       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mulps   xmm0,xmm0                       ; {AwAw AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwAw AwAw AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {AwAw AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {AwAw AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {AwAw AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        movaps  xmm1,xmm2
        movaps  xmm0,xmm2
        movaps  xmm3,xmm2

        cmpss   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpss   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtss xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulss   xmm0,xmm1       ; {  ... Ax*1/sqrt(Ax)}

        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_QVecMagnitudeFastAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QVecMagnitudeFastAsmSSE
vmp_QVecMagnitudeFastAsmSSE       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mulps   xmm0,xmm0                       ; {AwAw AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwAw AwAw AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {AwAw AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {AwAw AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {AwAw AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        movaps  xmm1,xmm2
        movaps  xmm0,xmm2
        movaps  xmm3,xmm2

        cmpss   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpss   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtss xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulss   xmm0,xmm1       ; {  ... Ax*1/sqrt(Ax)}

        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_QVecMagnitudeFastAsmSSE endp




QVsf3DX86       ends
        end

