        TITLE   Vsf3DX86.asm - 80(x)86 3D Math Special Function
        PAGE    53,160

;  Vsf3D.asm - 3D Math Special Functions
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8


        .XLIST

;;;        .387
;;;     .586P   .586    .686   .686P
;;;      .K3D    .XMM    .MMX

        .686
        .XMM

        .model flat, C

        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST


        .data
        align   16

ONES    REAL4   1.0,1.0,1.0,1.0
ONEHIGH REAL4   0.0,0.0,0.0,1.0

vTiny   REAL4   0.0000001,0.0000001,0.0000001,0.0000001
vZero   REAL4   0.0,0.0,0.0,0.0


;himsk32        OWORD   0FFFFFFFF000000000000000000000000h  ; FFFFFFFF 00000000 00000000 00000000 
;lomsk96        OWORD   000000000FFFFFFFFFFFFFFFFFFFFFFFFh      ; 00000000 FFFFFFFF FFFFFFFF FFFFFFFF
himsk32 DWORD   000000000h,000000000h,000000000h,0FFFFFFFFh
lomsk96 DWORD   0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,000000000h

tst REAL4 4.0,3.0,2.0,1.0
foo REAL4 0.0,0.0,0.0,0.0
foo2 REAL4 0.0,0.0,0.0,0.0


;------------------------------------------------------------------------------------------------


Vsf3DX86        SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Minimum
;
;
;void vmp_VecMin( vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB )
;{
;       pvD->x = (pvA->x < pvB->x) ? pvA->x : pvB->x;
;       pvD->y = (pvA->y < pvB->y) ? pvA->y : pvB->y;
;       pvD->z = (pvA->z < pvB->z) ? pvA->z : pvB->z;
;}

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecMinAsm3DNow
vmp_VecMinAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movq    mm0,[eax]                       ;vA.xy  {Ay   Ax}
        movd    mm1,(vmp3DVector PTR [eax]).z   ;vA.z   {0    Az}
        movq    mm2,[ecx]                       ;vB.xy  {By   Bx}
        movd    mm3,(vmp3DVector PTR [ecx]).z   ;vB.z   {0    Bz}

        pfmin   mm0,mm2                         ;    {Ay<By Ax<Bx}
        pfmin   mm1,mm3                         ;    { 0<0  Az<Bz}
        
        movq    [edx+0],mm0                     ;    {Miny Minx}
        movd    (vmp3DVector PTR [edx]).z,mm1   ;    {0    Minz}
;;;        femms
        ret
vmp_VecMinAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecMinAsmSSE0
vmp_VecMinAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movups  xmm0,[eax]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm1,[ecx]                      ;vB.xyz# {# Bz By Bx}
        movups  xmm2,[edx]

        minps   xmm0,xmm1       

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movups  [edx],xmm0                      ; Save Min vals
;;;        emms
        ret
vmp_VecMinAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecMinAsmSSE
vmp_VecMinAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movaps  xmm0,[eax]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm1,[ecx]                      ;vB.xyz# {# Bz By Bx}
        movaps  xmm2,[edx]

        minps   xmm0,xmm1       

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movaps  [edx],xmm0                      ; Save min vals
;;;        emms
        ret
vmp_VecMinAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Maximum
;
;
;void vmp_VecMax( vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB )
;{
;       pvD->x = (pvA->x > pvB->x) ? pvA->x : pvB->x;
;       pvD->y = (pvA->y > pvB->y) ? pvA->y : pvB->y;
;       pvD->z = (pvA->z > pvB->z) ? pvA->z : pvB->z;
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecMaxAsm3DNow
vmp_VecMaxAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movq    mm0,[eax]                       ;vA.xy  {Ay   Ax}
        movd    mm1,(vmp3DVector PTR [eax]).z   ;vA.z   {0    Az}
        movq    mm2,[ecx]                       ;vB.xy  {By   Bx}
        movd    mm3,(vmp3DVector PTR [ecx]).z   ;vB.z   {0    Bz}

        pfmax   mm0,mm2                         ;    {Ay>By Ax>Bx}
        pfmax   mm1,mm3                         ;    { 0>0  Az>Bz}
        
        movq    [edx+0],mm0                     ;    {Maxy Maxx}
        movd    (vmp3DVector PTR [edx]).z,mm1   ;    {0    Maxz}
;;;        femms
        ret
vmp_VecMaxAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecMaxAsmSSE0
vmp_VecMaxAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movups  xmm0,[eax]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm1,[ecx]                      ;vB.xyz# {# Bz By Bx}
        movups  xmm2,[edx]

        maxps   xmm0,xmm1       

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movups  [edx],xmm0                      ; Save max vals
;;;        emms
        ret
vmp_VecMaxAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecMaxAsmSSE
vmp_VecMaxAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     eax,vA                          ; Vector A
        mov     ecx,vB                          ; Vector B
        mov     edx,vD                          ; Vector Destination

        movaps  xmm0,[eax]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm1,[ecx]                      ;vB.xyz# {# Bz By Bx}
        movaps  xmm2,[edx]

        maxps   xmm0,xmm1       

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movaps  [edx],xmm0                      ; Save max vals
;;;        emms
        ret
vmp_VecMaxAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Square Root
;       (Similar to VecMagnitude)
;
;void vmp_VecSqrt(vmp3DVector * const pvD, const vmp3DVector * const pvA)
;{
;       assert(NULL!=pvD);
;       assert(NULL!=pvA);
;
;       assert(( pvA->x >= 0.0f ) && ( pvA->y >= 0.0f ) && ( pvA->z >= 0.0f ));
;
;       pvD->x = sqrtf( pvA->x );
;       pvD->y = sqrtf( pvA->y );
;       pvD->z = sqrtf( pvA->z );
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***     ***FAST***  15bit Precision
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecSqrtFastAsm3DNow
vmp_VecSqrtFastAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vD)
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DVector PTR [ecx]).z )

        movq    mm1,[ecx]                       ;vA.xy  {Ay   Ax}
        movd    mm6,[ecx+4]                     ;       {0    Ay}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm3,mm1              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Ax)  1/sqrt(Ax)}
        pfrsqrt     mm7,mm6                 ;                   {1/sqrt(Ay)  1/sqrt(Ay)}
        pfrsqrt     mm2,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Az)  1/sqrt(Az)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        punpckldq   mm3,mm7             ;            {1/sqrt(Ay)  1/sqrt(Ax)}

;       movq        mm4,mm2             ;            {1/sqrt(Az)  1/sqrt(Az)}
;       movq        mm5,mm3             ;            {1/sqrt(Ay)  1/sqrt(Ax)}
;       pfmul       mm3,mm3             ;X1=f(x0,x0) {1/Ay        1/Ax}
;       pfmul       mm2,mm2             ;X1=f(x0,x0) {1/Az        1/Az}

;       pfrsqit1    mm3,mm1             ;X2=f(x,x1)     {1st step}
;       pfrsqit1    mm2,mm0             ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit
;       pfrcpit2    mm3,mm5             ; y=f(x2,x0)    {2nd step}
;       pfrcpit2    mm2,mm4             ; y=f(x2,x0)    {2nd step}

        pfmul       mm1,mm3
        pfmul       mm0,mm2
        
        movq    [eax],mm1                       ;    {Ay/sqrt(Ay) Ax/sqrt(Ax)}
        movd    (vmp3DVector PTR [eax]).z,mm0   ;    {0           Az/sqrt(Az)}
;;;        femms
        ret
vmp_VecSqrtFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecSqrtAsm3DNow
vmp_VecSqrtAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vD)
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

ASSERT_FNEG( (vmp3DVector PTR [ecx]).x )
ASSERT_FNEG( (vmp3DVector PTR [ecx]).y )
ASSERT_FNEG( (vmp3DVector PTR [ecx]).z )

        movq    mm1,[ecx]                       ;vA.xy  {Ay   Ax}
        movd    mm6,[ecx+4]                     ;       {0    Ay}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm3,mm1              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Ax)  1/sqrt(Ax)}
        pfrsqrt     mm7,mm6                 ;                   {1/sqrt(Ay)  1/sqrt(Ay)}
        pfrsqrt     mm2,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(Az)  1/sqrt(Az)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        punpckldq   mm3,mm7             ;            {1/sqrt(Ay)  1/sqrt(Ax)}

        movq        mm4,mm2             ;            {1/sqrt(Az)  1/sqrt(Az)}
        movq        mm5,mm3             ;            {1/sqrt(Ay)  1/sqrt(Ax)}
        pfmul       mm3,mm3             ;X1=f(x0,x0) {1/Ay        1/Ax}
        pfmul       mm2,mm2             ;X1=f(x0,x0) {1/Az        1/Az}

        pfrsqit1    mm3,mm1             ;X2=f(x,x1)     {1st step}
        pfrsqit1    mm2,mm0             ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit
        pfrcpit2    mm3,mm5             ; y=f(x2,x0)    {2nd step}
        pfrcpit2    mm2,mm4             ; y=f(x2,x0)    {2nd step}

        pfmul       mm1,mm3
        pfmul       mm0,mm2
        
        movq    [eax],mm1                       ;    {rAy rAx}
        movd    (vmp3DVector PTR [eax]).z,mm0   ;    {0   rAz}
;;;        femms
        ret
vmp_VecSqrtAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecSqrtAsmSSE0
vmp_VecSqrtAsmSSE0       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination
        movups  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm2,[eax]

        sqrtps  xmm0,xmm1                       ; SqRoot {r# rAz rAy rAx}

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movups  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_VecSqrtAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecSqrtAsmSSE
vmp_VecSqrtAsmSSE       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination
        movaps  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm2,[eax]

        sqrtps  xmm0,xmm1                       ; SqRoot {r# rAz rAy rAx}

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movaps  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_VecSqrtAsmSSE endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecSqrtFastAsmSSE0
vmp_VecSqrtFastAsmSSE0       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination
        movups  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm2,[eax]

        movaps  xmm4,xmm1
        movaps  xmm3,xmm1
        movaps  xmm0,xmm1

        cmpps   xmm4,vTiny,5    ; >=  1's = #'s Okay
        cmpps   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtps xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm4       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulps   xmm0,xmm1       ; {Aw*1/sqrt(Aw) ... Ax*1/sqrt(Ax)}

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movups  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_VecSqrtFastAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecSqrtFastAsmSSE
vmp_VecSqrtFastAsmSSE       proc    near , vD:ptr, vA:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination
        movaps  xmm1,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm2,[eax]                      ;vD.###w {Dw #  #  #}

        movaps  xmm4,xmm1
        movaps  xmm3,xmm1
        movaps  xmm0,xmm1

        cmpps   xmm4,vTiny,5    ; >=  1's = #'s Okay
        cmpps   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtps xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm4       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulps   xmm0,xmm1       ; {... Az*1/sqrt(Az) ... Ax*1/sqrt(Ax)}

        andps   xmm2,OWORD PTR himsk32          ;        { #  0   0   0 }
        andps   xmm0,OWORD PTR lomsk96          ;        { 0 rAz rAy rAx}

        orps    xmm0,xmm2                       ;        { # rAz rAy rAx}
        movaps  [eax],xmm0                      ; Save Square Roots

;;;        emms
        ret
vmp_VecSqrtFastAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Normalization
;
;
;void vmp_VecNormalizeFast(vmp3DVector * const pvD, const vmp3DVector * const pvA)
;{
;       float fMag;
;       fMag = sqrtf( pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z );
;       if ( fMag < 0.0001f ) { // If too close to zero? Don't want a divide by zero!
;               pvD->x = pvA->x;  // Original values
;               pvD->y = pvA->y;
;               pvD->z = pvA->z;
;       } else {                        // Not zero so 1/n
;               fMag = 1.0f / fMag;
;               pvD->x = pvA->x * fMag;
;               pvD->y = pvA->y * fMag;
;               pvD->z = pvA->z * fMag;
;       }
;}
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!
;
;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecNormalizeFastAsm3DNow
vmp_VecNormalizeFastAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}
        movq    mm4,mm1                         ;       {Ay   Ax}
        movq    mm3,mm0                         ;       {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy                 fMag= r=AzAz+AyAy+AxAx

        ; too close zero ...???  1.0 / 10000.0 ???

        movd    ecx,mm0                         ; fMag
        cmp     ecx,FLOAT0001                   ; 0.0001f   3f800000h
        jl      short zmag                      ;just set vD=vA!!!

        ; Not too close so calculate...         r=AzAz+AyAy+AxAx

        pfrsqrt mm1,mm0                  ;X0=f(x)=m=1/sqrt(r)  {1/sqrt(r) 1/sqrt(r)}
;       pfmul    mm1,mm1                        ;X1=f(x0,x0)       {1/r  1/r}
        pfmul   mm4,mm1                         ;{Ny Nx}= {Ay/m Ax/m}
        pfmul   mm3,mm1                         ;{ 0 Nz}= { 0/m Az/m}

        ;       Save Resulting {x y z} Normals

zmag:   movq    [eax+0],mm4                             ;{Ny Nx}
        movd    (vmp3DVector PTR [eax]).z,mm3           ;{0  Nz}
        
;;;        femms
        ret
vmp_VecNormalizeFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecNormalizeAsm3DNow
vmp_VecNormalizeAsm3DNow       proc    near , vD:ptr, vA:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}
        movq    mm4,mm1                         ;       {Ay   Ax}
        movq    mm3,mm0                         ;       {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy

        ; too close zero ...???  1.0 / 10000.0 ???

        movd    ecx,mm0
        cmp     ecx,FLOAT0001                   ; 0.0001f
        jl      short zmag                      ;just set vD=vA!!!

        ; Not too close so calculate...         ;r=AzAz+AyAy+AxAx

        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x) {1/sqrt(r) 1/sqrt(r)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        movq        mm2,mm1             ;               {1/sqrt(r) 1/sqrt(r)}
        pfmul       mm1,mm1             ;X1=f(x0,x0)    {1/r 1/r}
        pfrsqit1    mm1,mm0             ;X2=f(x,x1)     {1st step}

         ; *** mm1 = Magnitude ***
         ;      Calculate sqrt() = (1/mag)  24bit
        pfrcpit2    mm1,mm2             ; y=f(x2,x0) {2nd step}  {# m}

        punpckldq mm1,mm1                               ;         {1/m   1/m}

        pfmul   mm4,mm1                         ;{Ny Nx}= {Ay/m Ax/m}
        pfmul   mm3,mm1                         ;{ 0 Nz}= { 0/m Az/m}

        ;       Save Resulting {x y z} Normals

zmag:   movq    [eax+0],mm4                             ;    {Ny Nx}
        movd    (vmp3DVector PTR [eax]).z,mm3           ;    {0  Nz}
        
;;;        femms
        ret
vmp_VecNormalizeAsm3DNow endp

;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecNormalizeAsmSSE0
vmp_VecNormalizeAsmSSE0       proc    near , vD:ptr, vA:ptr
        local uflow:REAL4

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm7,[eax]                      ;vD.xyz# {# ?  ?  ? }
        andps   xmm0,lomsk96                    ; Preserve original    {0 Az Ay Ax}
        andps   xmm7,himsk32                    ; Preserve destination {#  0  0  0}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAx AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???
        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        sqrtps  xmm0,xmm1       ; SqRoot  {1 rMz rMy rMx}
        divps   xmm6,xmm0                       ; n/m    {0/1 Az/#r Ay/#r Ax/#r}

zmag:   orps    xmm7,xmm6                       ;{# Nz Ny Nx}
        movups  [eax],xmm7                      ; Save xyz#

;;;        emms
        ret
vmp_VecNormalizeAsmSSE0 endp


;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecNormalizeAsmSSE
vmp_VecNormalizeAsmSSE       proc    near , vD:ptr, vA:ptr
        local uflow:DWORD

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm7,[eax]                      ;vD.xyz# {# ?  ?  ? }
        andps   xmm0,lomsk96                    ; Preserve original    {0 Az Ay Ax}
        andps   xmm7,himsk32                    ; Preserve destination {#  0  0  0}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAz AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???

        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        sqrtps  xmm0,xmm1                       ; SqRoot {1 rMz rMy rMx}
        divps   xmm6,xmm0                       ; n/m    {0 rNz rNy rNx}

zmag:   orps    xmm7,xmm6                       ;{# Nz Ny Nx}

        movaps  [eax],xmm7                      ; Save xyz#

;;;        emms
        ret
vmp_VecNormalizeAsmSSE endp



;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecNormalizeFastAsmSSE0
vmp_VecNormalizeFastAsmSSE0       proc    near , vD:ptr, vA:ptr
        local uflow:REAL4

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movups  xmm7,[eax]                      ;vD.xyz# {# ?  ?  ? }
        andps   xmm0,lomsk96                    ; Preserve original    {0 Az Ay Ax}
        andps   xmm7,himsk32                    ; Preserve destination {#  0  0  0}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAx AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???
        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        rsqrtps xmm0,xmm1                       ; SqRoot  {1/1     1/Mz      1/My      1/Mx}
        mulps   xmm6,xmm0                       ; n/m     {0   Az*(1/Mz) Ay*(1/My) Ax*(1/Mx)}

zmag:   orps    xmm7,xmm6                       ;{# Nz Ny Nx}
        movups  [eax],xmm7                      ; Save xyz#

;;;        emms
        ret
vmp_VecNormalizeFastAsmSSE0 endp


;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecNormalizeFastAsmSSE
vmp_VecNormalizeFastAsmSSE       proc    near , vD:ptr, vA:ptr
        local uflow:DWORD

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}
        movaps  xmm7,[eax]                      ;vD.xyz# {# ?  ?  ? }
        andps   xmm0,lomsk96                    ; Preserve original    {0 Az Ay Ax}
        andps   xmm7,himsk32                    ; Preserve destination {#  0  0  0}

        movaps  xmm6,xmm0                       ; {0   Az   Ay   Ax}
        mulps   xmm0,xmm0                       ; {0 AzAz AyAy AxAx}

        movaps  xmm1,xmm0                       ; {0 AzAz AyAy AxAx}
        movaps  xmm2,xmm0                       ; {0 AzAz AyAy AxAx}

        orps    xmm1,ONEHIGH                    ; {1 AzAz AyAy AxAx}

        shufps  xmm1,xmm1,11001001b     ;3 0 2 1        ; {1 AxAz AzAz AyAy}
        shufps  xmm2,xmm2,11010010b     ;3 1 0 2        ; {0 AyAy AxAx AzAz}

        addps   xmm1,xmm0                       ; {1+0 AzAz+AxAx AyAy+AzAz AxAx+AyAy}
        addps   xmm1,xmm2        ; {1+0+0 AzAz+AxAx+AyAy AyAy+AzAz+AxAx AxAx+AyAy+AzAz}

        ; Dot Product too close zero ...???  1.0 / 10000.0 ???

        movss   uflow,xmm1
        cmp     uflow,FLOAT0001                 ; 0.0001f    38d1b717h
        jl      short zmag                      ;just set vD=vA!!!

        ;       Calculate Square Root upon Dot Product

        rsqrtps xmm0,xmm1                       ; SqRoot  {1/1     1/Mz      1/My      1/Mx}
        mulps   xmm6,xmm0                       ; n/m     {0   Az*(1/Mz) Ay*(1/My) Ax*(1/Mx)}

zmag:   orps    xmm7,xmm6                       ;{# Nz Ny Nx}

        movaps  [eax],xmm7                      ; Save xyz#

;;;        emms
        ret
vmp_VecNormalizeFastAsmSSE endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Vector Magnitude - (3D Distance from {0,0,0})
;
;
;void vmp_VecMagnitude(float * const pfD, const vmp3DVector * const pvA)
;{
;       *pfD = sqrtf( pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z );
;}
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!
;
;_________________________________________________________________________________________________
; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecMagnitudeFastAsm3DNow
vmp_VecMagnitudeFastAsm3DNow       proc    near , pfD:ptr, pvA:ptr
        .K3D
;;;        femms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+BxAx AzAz+AyAy+AxAx}
                                                ;      r=AzAz+AyAy+AxAx
         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x)  {1/sqrt(r) 1/sqrt(r)}
        pfmul       mm0,mm1                     ;         {r/sqrt(r) r/sqrt(r)}
                                                ; r = r/sqrt(r)
        movd    [eax],mm0                       ; Save resulting distance float
        
;;;        femms
        ret
vmp_VecMagnitudeFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_VecMagnitudeAsm3DNow
vmp_VecMagnitudeAsm3DNow       proc    near , pfD:ptr, pvA:ptr
        .K3D
;;;        femms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; Vector Destination

        ;       Calculate XYZ product squares

        movq    mm1,[ecx]                       ;vA.x   {Ay   Ax}
        movd    mm0,(vmp3DVector PTR [ecx]).z   ;vA.z   {0    Az}
        pfmul   mm1,mm1                         ;       {AyAy AxAx}
        pfmul   mm0,mm0                         ;vA.z   {0    AzAz}

        pfacc   mm1,mm1                         ;       {  AyAy+AxAx      AyAy+AxAx}
        pfadd   mm0,mm1                         ;       {0+AyAy+AxAx AzAz+AyAy+AxAx}

         ;      Calculate Square Root upon Dot Product
         ;      pfrsqrt=15bit accuracy
        pfrsqrt     mm1,mm0              ;X0=f(x)=1/sqrt(x)  {#####  1/sqrt(AzAz+AyAy+AxAx)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        movq        mm2,mm1             ;            {#####  1/sqrt(AzAz+AyAy+AxAx)}
        pfmul       mm1,mm1             ;X1=f(x0,x0) {#####  1/(AzAz+AyAy+AxAx)}
        pfrsqit1    mm1,mm0             ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit
        pfrcpit2    mm1,mm2             ; y=f(x2,x0)    {2nd step}
        pfmul       mm0,mm1

        movd    [eax],mm0                       ; Save resulting distance float
        
;;;        femms
        ret
vmp_VecMagnitudeAsm3DNow endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecMagnitudeAsmSSE0
vmp_VecMagnitudeAsmSSE0       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        mulps   xmm0,xmm0                       ; {A#A# AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {A#A# A#A# AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {A#A# AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {A#A# AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {A#A# AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        sqrtss  xmm0,xmm2                       ; SqRoot
        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_VecMagnitudeAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecMagnitudeAsmSSE
vmp_VecMagnitudeAsmSSE       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        mulps   xmm0,xmm0                       ; {A#A# AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {A#A# A#A# AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {A#A# AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {A#A# AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {A#A# AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        sqrtss  xmm0,xmm2                       ; SqRoot
        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_VecMagnitudeAsmSSE endp



;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_VecMagnitudeFastAsmSSE0
vmp_VecMagnitudeFastAsmSSE0       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movups  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        mulps   xmm0,xmm0                       ; {A#A# AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {A#A# A#A# AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {A#A# AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {A#A# AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {A#A# AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        movaps  xmm1,xmm2
        movaps  xmm0,xmm2
        movaps  xmm3,xmm2

        cmpss   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpss   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtss xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulss   xmm0,xmm1       ; {  ... Ax*1/sqrt(Ax)}

        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_VecMagnitudeFastAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_VecMagnitudeFastAsmSSE
vmp_VecMagnitudeFastAsmSSE       proc    near , pfD:ptr, pvA:ptr
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_PTR(pvA)

        mov     ecx,pvA                         ; Vector A
        mov     eax,pfD                         ; float D

        ;       Calculate Dot Product

        movaps  xmm0,[ecx]                      ;vA.xyz# {# Az Ay Ax}

        mulps   xmm0,xmm0                       ; {A#A# AzAz AyAy AxAx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {A#A# A#A# AzAz AzAz}
        shufps  xmm1,xmm1,11100001b             ; {A#A# AzAz AxAx AyAy}

        addss   xmm2,xmm0                       ; {A#A# AzAz AxAx AzAz+AxAx}
        addss   xmm2,xmm1                       ; {A#A# AzAz AxAx AzAz+AxAx+AyAy}

        ;       Calculate Square Root upon Dot Product

        movaps  xmm1,xmm2
        movaps  xmm0,xmm2
        movaps  xmm3,xmm2

        cmpss   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpss   xmm3,vTiny,1    ; <   1's = too close to zero

        ;       vD = (1 / sqrt(vA))
        rsqrtss xmm0,xmm0

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulss   xmm0,xmm1       ; {  ... Ax*1/sqrt(Ax)}

        movss   [eax],xmm0                      ; Save 

;;;        emms
        ret
vmp_VecMagnitudeFastAsmSSE endp



Vsf3DX86        ends
        end

