
//
//  Bench.cpp - Vector 3D Trignometric Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Uses: chap03\ram     .\cpuid
//        chap04\vmp3d
//        chap08\fmd     .\qvmd3d
//        chap09\qvsf3d
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <memory.h>
#include    <math.h>
#include <stdio.h>

    //  Book Development Library Modules

#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"
#include "ram.h"

#include "VTrig3D.h"
#include "VTrig3DEx.h"
#include "QVmd3D.h"
#include "QVmd3DEx.h"
#include "QVsf3D.h"
#include "QVsf3DEx.h"
#include "Fmd.h"
#include "FmdEx.h"




typedef enum
{
    iTEST_FLT_SINCOS,
    iTEST_FLT_SIN,
    iTEST_FLT_COS,
    iTEST_FLT_TAN
} iTEST;




//
//  Vector angle to cosine calculation
//
void CosAng( float * const pfD, vmp3DQVector *pvA, vmp3DQVector *pvB )
{
    float fDot, fAMag, fBMag;

    ASSERT_PTR16(pvA);
    ASSERT_PTR16(pvB);
    ASSERT_PTR4(pfD);

    vmp_QDotProduct(&fDot, pvA, pvB);
    vmp_QVecMagnitude(&fAMag, pvA);
    vmp_QVecMagnitude(&fBMag, pvB);

    //  *pfD = fDot / (fAMag * fBMag);
    vmp_FMul( &fAMag, fAMag, fBMag );
    if ( fAMag < 0.0000001f )           // Too close to infinity
      {
        *pfD = fAMag;
      }
    else
      { 
        vmp_FDiv( pfD, fDot, fAMag );
      }
}



//
//  Test Cosine Angle
//
static void testCosAng( void )
{
    vmp3DQVector *pVec;
    float f, f2, fStep, fAngX, fAngX2, fAngCS, fAngT, fAngX0;
    float fCos, fSin;

    if (ramGet( (byte **)&pVec, sizeof(vmp3DQVector)*(2)) )     // Allocate memory
      {
        fStep = 1.0f;
        
        fAngX2 = DEG2RAD( 0.0f );

        pVec[0].x = cosf( fAngX2 );
        pVec[0].y = sinf( fAngX2 );
        pVec[0].z = 0.0f;
        pVec[0].w = 0.0f;

        for ( fAngX = -360.0f; fAngX <= 360.0f; fAngX += fStep )
          {
            fAngX2 = DEG2RAD( fAngX );

            fCos = cosf(fAngX2);
            fSin = sinf(fAngX2);

            pVec[1].x =  fCos;
            pVec[1].y =  fSin;
            pVec[1].z = 0.0f;
            pVec[1].w = 0.0f;

            CosAng( &f, &(pVec[0]), &(pVec[1]) );

            f2 = pVec[1].x;

            if (!vmp_IsFEqual( f2, f, SINGLE_PRECISION*10.0f ))
              {
                cout << f2 << " vs " << f << " failure! " << fAngX << endl;
              }

                //  Okay, we have an arcsine but which hemisphere? This is
                //  where tan2 is really handy!
            vmp_SIMDEntry();        // 1.#QNAN - MMX vs FPU toggle

            f = acos( fCos );
            fAngCS = (float) RAD2DEG( f );              // {0...180}
            if (asin(fSin) < 0.0)
              {
                fAngCS = -fAngCS;                           // {-180...180}
              }

            fAngT = (float) RAD2DEG( atan2( fSin, fCos ) );         // {-180...180}


                // Normalize our original angle to {-180...180}

            fAngX0 = (float) fmod( (double)fAngX, 360.0 );  //{-360...360} 
            if ( fAngX0 < -180.0f )
              {
                fAngX0 += 360.0f;
              }
            else if (fAngX0 > 180.0f)
              {
                fAngX0 -= 360.0f;
              }
            
                // Too much SPFP math so losing precsion!

            if (!vmp_IsFEqual( fAngCS, fAngX0, 0.001f ))
              {
                cout << fAngCS << " vs " << fAngX0 << " failure! " << endl;
              }

                // Too much SPFP math so losing precsion!

            if (!vmp_IsFEqual( fAngT, fAngX0, 0.001f ))
              {
                cout << fAngT << " vs " << fAngX0 << " failure! " << endl;
              }
          }

        ramRelease((byte **)&pVec);
      }
}


 
//
//  Test Trignometric Functions
//

static void testTrig( iTEST iFunc )
{
    float fStep, fAngX, fAngX2;
    float f, f2, e, e2;
    float fPrecision;
    bool bDual, bArc;

    fPrecision = 0.0000001f;
    fPrecision = 0.000001f;  //cossin sin cos
    fPrecision = 0.0001f;  //tan
    fStep = 10.0f;

        // Force proper alignment (Crude just for this demo)

    bArc = bDual = false;

    for ( fAngX = -360.0f; fAngX <= 360.0f; fAngX += fStep )
      {
        fAngX2 = DEG2RAD( fAngX ); /// 182.04f;

        switch(iFunc)
          {
            case iTEST_FLT_SINCOS:
                bDual = true;
                vmp_FSinCosGeneric( &f2, &e2, fAngX );
                vmp_SIMDEntry();
                vmp_FSinCos( &f, &e, fAngX );
                vmp_SIMDExit();
                break;

            case iTEST_FLT_SIN:
                bDual = false;
                bArc = true;
                vmp_FSinGeneric( &f2, fAngX );
                vmp_FASinGeneric( &e2, f2 );

                vmp_SIMDEntry();
                vmp_FSin( &f, fAngX );
                vmp_FASin( &e, f ); 
                vmp_SIMDExit();
                break;

            case iTEST_FLT_COS:
                bDual = false;
                vmp_FCosGeneric( &f2, fAngX );
                vmp_SIMDEntry();
                vmp_FCos( &f, fAngX );
                vmp_SIMDExit();
                break;

            case iTEST_FLT_TAN:
                bDual = false;
                vmp_FTanGeneric( &f2, fAngX );
                vmp_SIMDEntry();
                vmp_FTan( &f, fAngX );
                vmp_SIMDExit();
                break;
          }

        if (!vmp_IsFEqual( f2, f, fPrecision ))
          {
            cout << f2 << " vs " << f << " failure! " << fAngX << endl;
          }

        if (bArc)
          {
            if (!vmp_IsFEqual( e2, e, fPrecision ))
              {
                cout << e2 << " vs " << e << " failure! " << fAngX << endl;
              }
          }

        if (bDual)
          {
            if (!vmp_IsFEqual( e2, e, fPrecision ))
              {
                cout << e2 << " vs " << e << " failure! " << fAngX << endl;
              }
          }
      }
}





int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU

    cout << "\nVector Trignometry Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    VTrig3DSetup( &cinfo );                             // Setup our 3D Vector stuff
    QVmd3DSetup(  &cinfo );
    QVsf3DSetup(  &cinfo );
    FmdSetup(     &cinfo );

    cout << "Testing Trig - Sine:Cosine" << endl;
    testTrig( iTEST_FLT_SINCOS );

    cout << "Testing Trig - Sine" << endl;
    testTrig( iTEST_FLT_SIN );

    cout << "Testing Trig - Cosine" << endl;
    testTrig( iTEST_FLT_COS );

    cout << "Testing Trig - Tangent" << endl;
    testTrig( iTEST_FLT_TAN );

    cout << "Testing CosAng" << endl;
    testCosAng();

    cout << "Fin!\n\n" << flush;

    return 0;
}

