
//
//  Bench.cpp - Vector 3D Matrix Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      Hopefully by now you have realized the necessity of 16 byte aligned data
//      and so this program is written as such!
//
//      Because of all the Generic code there is an excessive amount of SIMD entry/exit
//      function calls. This gets dramatically reduced by
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <memory.h>
#include    <math.h>
//#include  <D3DX8.h>
#include    <D3dx8math.h>

    //  Book Development Library Modules

#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"
#include "Matrix3D.h"
#include "Matrix3DEx.h"

#include "vecBug.h"



#define STEP_VAL        10.0f



//
//  Matrix Racing - Direct X
//
static void raceMatrixDX( bool bAlign )
{
    float matrixbuf[(16)  *5 +3];   // enough space + 128bit alignment padding
    float fAngX, fAngY, fAngZ, fAngX2, fAngY2, fAngZ2, fStep, f;
    vmp3DMatrix *pmGRotX, *pmGRotY, *pmGRotZ;
    vmp3DMatrix *pmGTrans;
    vmp3DMatrix *pmGWorld;
    unsigned long t;

        //  Adjust Alignment factor

    if ( bAlign )
      {
        pmGRotX = (vmp3DMatrix*) ALIGN16((int)matrixbuf);
      }
    else        // unaligned
      {
        pmGRotX = (vmp3DMatrix*) matrixbuf;
        if (4 != (0x0007 & ((int)pmGRotX)) )    // Force unalignment
          {
            pmGRotX = (vmp3DMatrix*) (matrixbuf+1);
          }
      }

    pmGRotY = pmGRotX + 1;
    pmGRotZ = pmGRotY + 1;
    pmGWorld = pmGRotZ + 1;
    pmGTrans = pmGWorld + 1;

    fStep = STEP_VAL;

    CpuDelaySet();

    for ( fAngX = -360.0f; fAngX <= 360.0f; fAngX += fStep )
      {
        for ( fAngY = -180.0f; fAngY <= 180.0f; fAngY += fStep )
          {
            for ( fAngZ = -360.0f; fAngZ <= 360.0f; fAngZ += fStep )
              {
                fAngX2 = DEG2RAD( fAngX );
                fAngY2 = DEG2RAD( fAngY );
                fAngZ2 = DEG2RAD( fAngZ );

                D3DXMatrixRotationX( (D3DXMATRIX *)pmGRotX, fAngX2 );
                D3DXMatrixRotationY( (D3DXMATRIX *)pmGRotY, fAngY2 );
                D3DXMatrixRotationZ( (D3DXMATRIX *)pmGRotZ, fAngZ2 );

                D3DXMatrixTranslation( (D3DXMATRIX *)pmGTrans, fAngX, fAngY, fAngZ );       

                vmp_MatrixTestFill( *pmGWorld );

                D3DXMatrixMultiply( (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGRotX );
                D3DXMatrixMultiply( (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGRotY );
                D3DXMatrixMultiply( (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGRotZ );
                D3DXMatrixMultiply( (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGWorld, (D3DXMATRIX *)pmGTrans );
              }
          }
      }

    t = CpuDelayCalc();
    f = ((float)t) / 1000000;

    cout << "  Time: " << t << " (" << f << "K) cycles.\n" << endl;
}



//
//  Matrix Racing - Generic 'C'
//
static void raceMatrixGeneric( bool bAlign )
{
    float matrixbuf[(16)  *5 +3];   // enough space + 128bit alignment padding
    float f, fAngX, fAngY, fAngZ, fAngX2, fAngY2, fAngZ2, fStep;
    vmp3DMatrix *pmGRotX, *pmGRotY, *pmGRotZ;
    vmp3DMatrix *pmGTrans;
    vmp3DMatrix *pmGWorld;
    unsigned long t;

        //  Adjust Alignment factor

    if ( bAlign )
      {
        pmGRotX = (vmp3DMatrix*) ALIGN16((int)matrixbuf);
      }
    else        // unaligned
      {
        pmGRotX = (vmp3DMatrix*) matrixbuf;
        if (4 != (0x0007 & ((int)pmGRotX)) )    // Force unalignment
          {
            pmGRotX = (vmp3DMatrix*) (matrixbuf+1);
          }
      }

    pmGRotY = pmGRotX + 1;
    pmGRotZ = pmGRotY + 1;
    pmGWorld = pmGRotZ + 1;
    pmGTrans = pmGWorld + 1;

    fStep = STEP_VAL;

    CpuDelaySet();

    for ( fAngX = -360.0f; fAngX <= 360.0f; fAngX += fStep )
      {
        for ( fAngY = -180.0f; fAngY <= 180.0f; fAngY += fStep )
          {
            for ( fAngZ = -360.0f; fAngZ <= 360.0f; fAngZ += fStep )
              {
                fAngX2 = DEG2RAD( fAngX );
                fAngY2 = DEG2RAD( fAngY );
                fAngZ2 = DEG2RAD( fAngZ );

                vmp_MatrixSetRotateXGeneric( *pmGRotX, fAngX2 );
                vmp_MatrixSetRotateYGeneric( *pmGRotY, fAngY2 );
                vmp_MatrixSetRotateZGeneric( *pmGRotZ, fAngZ2 );
                vmp_MatrixSetTranslateGeneric( *pmGTrans, fAngX, fAngY, fAngZ );

                vmp_MatrixTestFill( *pmGWorld );

                vmp_MatrixMulGeneric( *pmGWorld, *pmGWorld, *pmGRotY );
                vmp_MatrixMulGeneric( *pmGWorld, *pmGWorld, *pmGRotY );
                vmp_MatrixMulGeneric( *pmGWorld, *pmGWorld, *pmGRotZ );
                vmp_MatrixMulGeneric( *pmGWorld, *pmGWorld, *pmGTrans );
              }
          }
      }

    t =CpuDelayCalc();
    f = ((float)t) / 1000000;

    cout << "  Time: " << t << " (" << f << "K) cycles.\n" << endl;
}



//
//  Matrix Racing - Generic 'Asm'
//
static void raceMatrix( CpuInfo *pcinfo )
{
    float matrixbuf[(16)  *5 +3];   // enough space + 128bit alignment padding
    float f, fAngX, fAngY, fAngZ, fAngX2, fAngY2, fAngZ2, fStep;
    vmp3DMatrix *pmGRotX, *pmGRotY, *pmGRotZ;
    vmp3DMatrix *pmGTrans;
    vmp3DMatrix *pmGWorld;
    unsigned long t;
    bool bSIMD;

    bSIMD = false;

    if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        bSIMD = true;
      }

        //  Adjust Alignment factor

    pmGRotX = (vmp3DMatrix*) ALIGN16((int)matrixbuf);
    pmGRotY = pmGRotX + 1;
    pmGRotZ = pmGRotY + 1;
    pmGWorld = pmGRotZ + 1;
    pmGTrans = pmGWorld + 1;

    fStep = STEP_VAL;

    CpuDelaySet();

    for ( fAngX = -360.0f; fAngX <= 360.0f; fAngX += fStep )
      {
        for ( fAngY = -180.0f; fAngY <= 180.0f; fAngY += fStep )
          {
            for ( fAngZ = -360.0f; fAngZ <= 360.0f; fAngZ += fStep )
              {
                fAngX2 = DEG2RAD( fAngX );
                fAngY2 = DEG2RAD( fAngY );
                fAngZ2 = DEG2RAD( fAngZ );

                    // FPU
                vmp_MatrixSetRotateX( *pmGRotX, fAngX2 );
                vmp_MatrixSetRotateY( *pmGRotY, fAngY2 );
                vmp_MatrixSetRotateZ( *pmGRotZ, fAngZ2 );
                vmp_MatrixSetTranslate( *pmGTrans, fAngX, fAngY, fAngZ );

                vmp_MatrixTestFill( *pmGWorld );

                    // 3DNow = MMX else FPU or XMM

                if (bSIMD)
                  {
                    vmp_SIMDEntry();
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotY );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotY );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotZ );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGTrans );
                    vmp_SIMDExit();
                  }
                else
                  {
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotY );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotY );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGRotZ );
                    vmp_MatrixMul( *pmGWorld, *pmGWorld, *pmGTrans );
                  }
              }
          }
      }

    t =CpuDelayCalc();
    f = ((float)t) / 1000000;

    cout << "  Time: " << t << " (" << f << "K) cycles.\n" << endl;
}



int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU

    cout << "\nVector Matrix Race Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    if (!( CPUBITS_MMX & cinfo.nBits ))
      {
        cout << "ERROR! MMX Not supported!\n\n" << flush;
        return -1;
      }

    Matrix3DSetup( &cinfo );                                // Setup our 3D Vector stuff

    cout << "Matrix DirectX (aligned)" << endl;
    raceMatrixDX( true );

    cout << "Matrix DirectX (unaligned)" << endl;
    raceMatrixDX( false );

    cout << "Matrix Generic 'C' (aligned)" << endl;
    raceMatrixGeneric( true );

    cout << "Matrix Generic 'C' (unaligned)" << endl;
    raceMatrixGeneric( false );

    cout << "Matrix Custom Asm (aligned)" << endl;
    raceMatrix( &cinfo );

    cout << "NOTE:  Custom Asm (unaligned) code was purposely NOT written!!!\n\n";
    cout << "       Note that DirectX's matrix libraries are optimized and the\n";
    cout << "       assembly libraries used here are not!  The biggest lesson\n";
    cout << "       here is to note the aligned versus unaligned speeds when\n";
    cout << "       calling functions such as with DirectX!\n" << endl;

    cout << "Fin!\n\n" << flush;

    return 0;
}

