        TITLE   Matrig3DX86M.asm - 80(x)86 3D Math Matrix Trigonometry Function
        PAGE    53,160

;  MaTrig3DX86.asm - 3D Matrix Trigonometric Math
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

        .686
        .XMM

        .model flat, C

 
        include KariType.inc
        include vmp3DX86M.inc

        .LIST


OFLOW_TEST      equ     1       ; Overflow testing & fixup




        .data
        align   16



;------------------------------------------------------------------------------------------------


MaTrig3DX86     SEGMENT USE32 PUBLIC 'CODE'

;_________________________________________________________________________________________________
;=================================================================================================
;
;       Fixup Real4 (float)
;
;        eax=bad float
; NOTE: ST(0) already contains the float!!!

if OFLOW_TEST eq 1
FixupReal4 proc near
        fstp    qword ptr [esp-8-10]            ;Save Real8 (double) & pop ST(0)

                ; Build a Real10

        mov     eax,[esp-8-10+4]                ; Hi
        or      eax,7FFF0000h           ;Set Real8 (Double)
        mov     dword ptr [esp-10+6],eax        ;7FFFXXXX????????????

        mov     eax,[esp-8-10+4]                ; Real8 Hi
        mov     ecx,[esp-8-10+0]                ; Real8 Lo
        shld    eax,ecx,11
        shl     ecx,11
        mov     dword ptr [esp-10+4],eax        ;7FFFHHHHHHHH????????
        mov     dword ptr [esp-10+0],ecx        ;7FFFHHHHHHHHLLLLLLLL
        fld     tbyte ptr [esp-10+0]    ;Load Real10

; FIXME - Overflow handling!!!

        ret    ;ST(0)=value
FixupReal4 endp
endif


;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Sine - Cosine
;
;void vmp_FSinCos( float * const pfSin, float * const pfCos, float fRads )
;{
;       ASSERT_PTR4( pfSin );
;       ASSERT_PTR4( pfCos );
;
;       *pfSin = sinf( fRads );
;       *pfCos = cosf( fRads );
;}
;_________________________________________________________________________________________________

        public  vmp_FSinCosAsmFPU
vmp_FSinCosAsmFPU       proc    near, pfSin:ptr, pfCos:ptr, fRads:REAL4

ASSERT_PTR4( pfSin )
ASSERT_PTR4( pfCos )

        fld     fRads

        OFLOW_FIXUP fRads                               ; Overflow fixup (optional)

        fwait
        fsincos                                 ;ST(0)=cos ST(1)=sin

        mov     eax,pfSin
        mov     ecx,pfCos

        fwait

        fstp    (REAL4 PTR [ecx])                       ;ST(0)=cos ST(1)=sin
        fstp    (REAL4 PTR [eax])                       ;sin

        ret
vmp_FSinCosAsmFPU endp




;=================================================================================================
;       Matrix Set X Rotation
;
;void vmp_MatrixSetRotateX( vmp3DMatrix * const pMat, float fRads )
;
;       Builds a 4x4 X rotation matrix
;
;     | 1   0   0   0 |
;     | 0  cos sin  0 |
;     | 0 -sin cos  0 |
;     | 0   0   0   1 |
;
;_________________________________________________________________________________________________

        public  vmp_MatrixSetRotateXAsmFPU
vmp_MatrixSetRotateXAsmFPU       proc    near , pMat:ptr, fRads:REAL4

ASSERT_PTR4( pMat )

        fld     fRads

        OFLOW_FIXUP fRads                               ; Overflow fixup (optional)

        mov     eax,pMat                                ; Matrix
        mov     ecx,FLOAT1                      ; 1.0f
        mov     edx,FLOAT0                      ; 0.0f

        fsincos                                 ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;1.0
        mov     (vmp3DMatrix PTR [eax])._m01,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m02,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;0.0

        fwait
        fst     (vmp3DMatrix PTR [eax])._m11    ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m10,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;0.0

        fstp    (vmp3DMatrix PTR [eax])._m22    ;     =cos ST(0)=sin

        mov     (vmp3DMatrix PTR [eax])._m20,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;0.0

        fst     (vmp3DMatrix PTR [eax])._m12        ;sin

        mov     (vmp3DMatrix PTR [eax])._m30,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;0.0

        fchs                                    ;-sin

        mov     (vmp3DMatrix PTR [eax])._m32,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;1.0

        fwait
        fstp    (vmp3DMatrix PTR [eax])._m21        ;-sin  ST(0)=sin

        ret
vmp_MatrixSetRotateXAsmFPU endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix Set Y Rotation
;
;void vmp_MatrixSetRotateY( vmp3DMatrix * const pMat, float fRads )
;
;       Builds a 4x4 Y rotation matrix
;
;     | cos  0 -sin  0 |
;     |  0   1   0   0 |
;     | sin  0  cos  0 |
;     |  0   0   0   1 |
;
;_________________________________________________________________________________________________

        public  vmp_MatrixSetRotateYAsmFPU
vmp_MatrixSetRotateYAsmFPU       proc    near , pMat:ptr, fRads:REAL4

ASSERT_PTR4( pMat )

        fld     fRads

        OFLOW_FIXUP fRads                               ; Overflow fixup (optional)

        mov     eax,pMat                                ; Matrix
        mov     ecx,FLOAT1                      ; 1.0f
        mov     edx,FLOAT0                      ; 0.0f

        fsincos                                 ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m01,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m10,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m11,ecx        ;1.0

        fwait
        fst     (vmp3DMatrix PTR [eax])._m00    ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m12,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;0.0

        fstp    (vmp3DMatrix PTR [eax])._m22    ;     =cos ST(0)=sin

        mov     (vmp3DMatrix PTR [eax])._m21,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;0.0

        fst     (vmp3DMatrix PTR [eax])._m20        ;sin

        mov     (vmp3DMatrix PTR [eax])._m30,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;0.0

        fchs                                    ;-sin

        mov     (vmp3DMatrix PTR [eax])._m32,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;1.0

        fwait
        fstp    (vmp3DMatrix PTR [eax])._m02        ;-sin  ST(0)=sin

        ret
vmp_MatrixSetRotateYAsmFPU endp


;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix Set Z Rotation
;
;void vmp_MatrixSetRotateZ( vmp3DMatrix * const pMat, float fRads )
;
;       Builds a 4x4 Z rotation matrix
;
;     |  cos sin  0   0 |
;     | -sin cos  0   0 |
;     |   0   0   1   0 |
;     |   0   0   0   1 |
;
;_________________________________________________________________________________________________

        public  vmp_MatrixSetRotateZAsmFPU
vmp_MatrixSetRotateZAsmFPU       proc    near , pMat:ptr, fRads:REAL4

ASSERT_PTR4( pMat )

        fld     fRads

        OFLOW_FIXUP fRads                               ; Overflow fixup (optional)

        mov     eax,pMat                                ; Matrix
        mov     ecx,FLOAT1                      ; 1.0f
        mov     edx,FLOAT0                      ; 0.0f

        fsincos                                 ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m02,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m12,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;0.0

        fwait
        fst     (vmp3DMatrix PTR [eax])._m00    ;ST(0)=cos ST(1)=sin

        mov     (vmp3DMatrix PTR [eax])._m20,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m21,edx        ;0.0

        fstp    (vmp3DMatrix PTR [eax])._m11    ;     =cos ST(0)=sin

        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;1.0
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;0.0

        fst     (vmp3DMatrix PTR [eax])._m01        ;sin

        mov     (vmp3DMatrix PTR [eax])._m30,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;0.0

        fchs                                    ;-sin

        mov     (vmp3DMatrix PTR [eax])._m32,edx        ;0.0
        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;1.0

        fwait
        fstp    (vmp3DMatrix PTR [eax])._m10        ;-sin  ST(0)=sin

        ret
vmp_MatrixSetRotateZAsmFPU endp



MaTrig3DX86     ends
        end

