  
//
//  Matrix3DEx.h - 3D Math Matrix Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to accomodate the book.
//
//      This module is designed for fast 3D math
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Kari Games Master Definitions
#include    "CpuAsm.h"
#include    "vmp3D.h"


#ifndef __Matrix3DEx_H
#define __Matrix3DEx_H

#ifdef __cplusplus
extern "C" {
#endif


void Matrix3DSetup( const CpuInfo * const pcinfo );

void vmp_MatrixSetIdentityGeneric( vmp3DMatrix pMatrix );
void vmp_MatrixSetIdentityAsm(     vmp3DMatrix pMatrix );
void vmp_MatrixSetIdentityAsmMMX(  vmp3DMatrix pMatrix );
void vmp_MatrixSetIdentityAsmSSE(  vmp3DMatrix pMatrix );

void vmp_MatrixSetScalingGeneric( vmp3DMatrix pM, float fSx, float fSy, float fSz );
void vmp_MatrixSetScalingAsm(     vmp3DMatrix pM, float fSx, float fSy, float fSz );
void vmp_MatrixSetScalingAsmMMX(  vmp3DMatrix pM, float fSx, float fSy, float fSz );
void vmp_MatrixSetScalingAsmSSE(  vmp3DMatrix pM, float fSx, float fSy, float fSz );

void vmp_MatrixTransposeGeneric( vmp3DMatrix dMx, const vmp3DMatrix aMx );
void vmp_MatrixTransposeAsm(     vmp3DMatrix dMx, const vmp3DMatrix aMx );
void vmp_MatrixTransposeAsmSSE(  vmp3DMatrix dMx, const vmp3DMatrix aMx );

void vmp_MatrixSetTranslateGeneric( vmp3DMatrix pM, float fTx, float fTy, float fTz );
void vmp_MatrixSetTranslateAsm(     vmp3DMatrix pM, float fTx, float fTy, float fTz );
void vmp_MatrixSetTranslateAsmMMX(  vmp3DMatrix pM, float fTx, float fTy, float fTz );
void vmp_MatrixSetTranslateAsmSSE(  vmp3DMatrix pM, float fTx, float fTy, float fTz );

void vmp_MatrixSetRotateXGeneric( vmp3DMatrix pM, float fRads );
void vmp_MatrixSetRotateXAsmFPU(  vmp3DMatrix pM, float fRads );
                                              
void vmp_MatrixSetRotateYGeneric( vmp3DMatrix pM, float fRads );
void vmp_MatrixSetRotateYAsmFPU(  vmp3DMatrix pM, float fRads );

void vmp_MatrixSetRotateZGeneric( vmp3DMatrix pM, float fRads );
void vmp_MatrixSetRotateZAsmFPU(  vmp3DMatrix pM, float fRads );


void vmp_MatrixCopyGeneric( vmp3DMatrix dMx, const vmp3DMatrix aMx );
void vmp_MatrixCopyAsm(     vmp3DMatrix dMx, const vmp3DMatrix aMx );
void vmp_MatrixCopyAsmSSE(  vmp3DMatrix dMx, const vmp3DMatrix aMx );
void vmp_MatrixCopyAsmSSE0( vmp3DMatrix dMx, const vmp3DMatrix aMx );


void vmp_MatrixAddGeneric(  vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixAddAsm3DNow( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixAddAsmSSE(   vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );


void vmp_MatrixMulGenericSlow( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixMulGeneric(     vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixMulAsm(         vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixMulAsm3DNow(    vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
void vmp_MatrixMulAsmSSE(      vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );

bool vmp_MatrixInvGeneric( vmp3DMatrix dMx, const vmp3DMatrix aMx );

#ifdef __cplusplus
}
#endif

#endif  //__Matrix3DEx_H


