
//
//  Matrix3DX86.cpp - 3D Vector Matrix Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <math.h>

    //  Book Development Library Modules

#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "Matrix3D.h"                                    // 3D Math
#include    "Matrix3DEx.h"




//
//  Setup Matrix3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void Matrix3DSetup( const CpuInfo * const pcinfo )
{
    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualAsmSSE;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualGeneric;   //AsmSSE;
 
        vmp_MatrixSetIdentity =     vmp_MatrixSetIdentityAsmSSE;
        vmp_MatrixSetScaling =      vmp_MatrixSetScalingAsmSSE;
        vmp_MatrixSetTranslate =    vmp_MatrixSetTranslateAsmSSE;

        vmp_MatrixSetRotateX =      vmp_MatrixSetRotateXAsmFPU;
        vmp_MatrixSetRotateY =      vmp_MatrixSetRotateYAsmFPU;
        vmp_MatrixSetRotateZ =      vmp_MatrixSetRotateZAsmFPU;

        vmp_MatrixTranspose =       vmp_MatrixTransposeAsmSSE;
        vmp_MatrixCopy =            vmp_MatrixCopyAsmSSE;
        vmp_MatrixAdd =             vmp_MatrixAddAsmSSE;
        vmp_MatrixMul =             vmp_MatrixMulAsmSSE;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualAsm3DNow;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualAsm3DNow;
 
        vmp_MatrixSetIdentity =     vmp_MatrixSetIdentityAsmMMX;
        vmp_MatrixSetScaling =      vmp_MatrixSetScalingAsmMMX;
        vmp_MatrixSetTranslate =    vmp_MatrixSetTranslateAsmMMX;

        vmp_MatrixSetRotateX =      vmp_MatrixSetRotateXAsmFPU;
        vmp_MatrixSetRotateY =      vmp_MatrixSetRotateYAsmFPU;
        vmp_MatrixSetRotateZ =      vmp_MatrixSetRotateZAsmFPU;

        vmp_MatrixTranspose =       vmp_MatrixTransposeAsm;
        vmp_MatrixCopy =            vmp_MatrixCopyAsm;
        vmp_MatrixAdd =             vmp_MatrixAddAsm3DNow;
        vmp_MatrixMul =             vmp_MatrixMulAsm3DNow;
      }
    
        // Don't use MMX versions, only generic! Prevent EMMS!

    else if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualGeneric;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualGeneric;
 
        vmp_MatrixSetIdentity =     vmp_MatrixSetIdentityAsm;
        vmp_MatrixSetScaling =      vmp_MatrixSetScalingAsm;
        vmp_MatrixSetTranslate =    vmp_MatrixSetTranslateAsm;

        vmp_MatrixSetRotateX =      vmp_MatrixSetRotateXAsmFPU;
        vmp_MatrixSetRotateY =      vmp_MatrixSetRotateYAsmFPU;
        vmp_MatrixSetRotateZ =      vmp_MatrixSetRotateZAsmFPU;

        vmp_MatrixTranspose =       vmp_MatrixTransposeGeneric;
        vmp_MatrixCopy =            vmp_MatrixCopyAsm;
        vmp_MatrixAdd =             vmp_MatrixAddGeneric;
        vmp_MatrixMul =             vmp_MatrixMulGeneric;

      }

    else
      {
        ASSERT(0);
      }
}


