
//
//  Quat3D.cpp - Quanternion Functions
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 30 Apr 2002 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <math.h>
#include <stdio.h>

    //  Book Development Library Modules

#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "Matrix3D.h"                                    // 3D Math
#include    "Matrix3DEx.h"
#include    "Quat3D.h"
#include    "Quat3DEx.h"
#include    "vecBug.h"

//  uncomment to use functions equivalent of DirectX library D3Dx8.lib
//#define DIRECTX_EQUIV



vmp_IsQuatEqualProc         vmp_IsQuatEqual;

vmp_QuatCopyProc            vmp_QuatCopy;
vmp_QuatConjugateProc       vmp_QuatConjugate;
vmp_QuatInvProc             vmp_QuatInv;
vmp_QuatSquareProc          vmp_QuatSquare;
vmp_QuatSqrtProc            vmp_QuatSqrt;
vmp_QuatDotProductProc      vmp_QuatDotProduct;
vmp_QuatMagnitudeProc       vmp_QuatMagnitude;
vmp_QuatMagnitudeProc       vmp_QuatMagnitudeFast;
vmp_QuatNormalizeProc       vmp_QuatNormalize;
vmp_QuatNormalizeProc       vmp_QuatNormalizeFast;
vmp_QuatAddProc             vmp_QuatAdd;
vmp_QuatSubProc             vmp_QuatSub;
vmp_QuatNegProc             vmp_QuatNeg;
vmp_QuatRotAxisProc         vmp_QuatRotAxis;
vmp_QuatToRotAxisProc       vmp_QuatToRotAxis;
vmp_QuatRotYawPitchRollProc vmp_QuatRotYawPitchRoll;
vmp_QuatMulProc             vmp_QuatMul;
vmp_QuatDivProc             vmp_QuatDiv;
vmp_QuatExpProc             vmp_QuatExp;
vmp_QuatLnProc              vmp_QuatLn;
vmp_QuatToMatrixProc        vmp_QuatToMatrix;
vmp_QuatMatrixProc          vmp_QuatMatrix;
vmp_QuatSlerpProc           vmp_QuatSlerp;





//
//  Compare two Quaternions
//

bool vmp_IsQuatEqualGeneric( const vmpQuat * const pqA, const vmpQuat * const pqB, float fPrec )
{
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    if (   !vmp_IsFEqual(pqA->x, pqB->x, fPrec) 
        || !vmp_IsFEqual(pqA->y, pqB->y, fPrec) 
        || !vmp_IsFEqual(pqA->z, pqB->z, fPrec) 
        || !vmp_IsFEqual(pqA->w, pqB->w, fPrec))
      {
            // No match (Maybe vector XYZ pointed in opposite direction with opposite rotation W?)

        if (   !vmp_IsFEqual(-pqA->x, pqB->x, fPrec) 
            || !vmp_IsFEqual(-pqA->y, pqB->y, fPrec) 
            || !vmp_IsFEqual(-pqA->z, pqB->z, fPrec) 
            || !vmp_IsFEqual(-pqA->w, pqB->w, fPrec))
          {
            return false;
          }
      }

    return true;
}                                               




//
//  Quaternion Copy
//

void vmp_QuatCopyGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    pqD->x = pqA->x;
    pqD->y = pqA->y;
    pqD->z = pqA->z;
    pqD->w = pqA->w;
}



//
//  Quaternion DotProduct
//
//  Quaternion must be normalized!
//
//  Same as DirectX - D3DXQuaternionDot
//

void vmp_QuatDotProductGeneric(float * const pfD, const vmpQuat * const pqA, const vmpQuat * const pqB)
{
    ASSERT_PTR4(pfD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    *pfD = (pqA->x * pqB->x + pqA->y * pqB->y + pqA->z * pqB->z + pqA->w * pqB->w);
}



//
//  Quaternion Magnitude
//
//  Same as DirectX - D3DXQuaternionLength
//

void vmp_QuatMagnitudeGeneric(float * const pfD, const vmpQuat * const pqA)
{
    ASSERT_PTR4(pfD);
    ASSERT_PTR4(pqA);

    *pfD = sqrtf(pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z + pqA->w * pqA->w);
}



//
//  Quaternion Normalization
//
//  Same as DirectX - D3DXQuaternionNormalize
//

void vmp_QuatNormalizeGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    float fLen;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

        //  Sum of squares
    fLen = pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z + pqA->w * pqA->w;

    if (SINGLE_PRECISION <= fLen)   // Not too close to zero?
      {
        fLen = 1.0f / sqrtf(fLen);
        pqD->x = pqA->x * fLen;
        pqD->y = pqA->y * fLen;
        pqD->z = pqA->z * fLen;
        pqD->w = pqA->w * fLen;
      }
    else                    // Really close to zero
      {                     // Normal means sqrt(xx+yy+zz+ww)=1 so Microsoft
        pqD->x = pqA->x;    // Forces .W to 1 with the correct sign!
        pqD->y = pqA->y;
        pqD->z = pqA->z;
        pqD->w = 1.0f;

#if 01      // Clone sign bit!
        *(uint32*)&(pqD->w) |= 0x80000000 & *(uint32 *)&(pqA->w);
#else
        if (0.0f > pqA->w) pqD->w = -1.0f;  // Need signed 1 or -1
#endif
      }
}



//
//  Quaternion Summation
//

void vmp_QuatAddGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB)
{
    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    pqD->x = pqA->x + pqB->x;
    pqD->y = pqA->y + pqB->y;
    pqD->z = pqA->z + pqB->z;
    pqD->w = pqA->w + pqB->w;
}



//
//  Quaternion Difference
//

void vmp_QuatSubGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB)
{
    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    pqD->x = pqA->x - pqB->x;
    pqD->y = pqA->y - pqB->y;
    pqD->z = pqA->z - pqB->z;
    pqD->w = pqA->w - pqB->w;
}



//
//  Convert a 'Normalized' axis and 'radian' angle to quaternions
//
//  Same as DirectX - D3DXQuaternionRotationAxis (with DIRECTX_EQUIV)
//

void vmp_QuatRotAxisGeneric( vmpQuat * const pqD, const vmp3DVector * const pvA, float fAngle )
{
    float f;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pvA);

#ifndef DIRECTX_EQUIV
        //  Cos is 180 degrees {-PI...0...PI}. Divide by 2 halves 360 degrees
    pqD->w = 0.5f * cosf(fAngle);

        // Quad Normalization (sum of squares)
    f = (pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z);

        // NOTE: a normalized vector with a magnitude of zero is technically undefined
        //      so handle it!

    if (SINGLE_PRECISION <= f)  // Not too close to zero?
      { 
        f = 1.0f / sqrtf(f);

        pqD->x = pvA->x * f;
        pqD->y = pvA->y * f;
        pqD->z = pvA->z * f;
      }
    else                            // Too close to zero thus infinity!
      {                             // thus the effect is negligible upon the Vector component!
        pqD->x = pvA->x;
        pqD->y = pvA->y;
        pqD->z = pvA->z;
      }
#else   // DIRECTX equivalent
        //  Cos is 180 degrees {-PI...0...PI}. Divide by 2 halves 360 degrees
    vmp_FSinCos(&f, &(pvD->w), fAngle * 0.5f);

        //  pvA is Normalized
    pqD->x = f * pvA->x;
    pqD->y = f * pvA->y;
    pqD->z = f * pvA->z;
#endif
}



//
//  Convert a (Unit) quaternion to a normalized axis
//
//  Same as DirectX - D3DXQuaternionToAxisAngle (with DIRECTX_EQUIV)
//
//  Needs Unit Quaternion

void vmp_QuatToRotAxisGeneric( vmp3DVector * const pvD, float * const pfAngle, const vmpQuat * const pqA )
{
    float fLen;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pfAngle);

        // Correct back to true 360 degree angle in radians
    *pfAngle = 2.0f * acosf(pqA->w);            // radian/2

        // Quad Normalization (sum of squares)

#ifndef DIRECTX_EQUIV
    fLen = (pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z);
#else   // DIRECTX equivalent
    fLen = (pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z + pqA->w * pqA->w);
#endif
    
    if (SINGLE_PRECISION <= fLen)   // Not too close to zero?
      {
        fLen = 1.0f / sqrtf(fLen);

        pvD->x = pqA->x * fLen;     // Quaternions back to radians
        pvD->y = pqA->y * fLen;
        pvD->z = pqA->z * fLen;
      }
    else                            // Too close to zero thus infinity!
      {                             // thus the effect is negligible upon the Vector component!
        pvD->x = pqA->x;
        pvD->y = pqA->y;
        pvD->z = pqA->z;
      }
}



//
//  Quaternion Conjugate
//
//  d = a1-a2i-a3j-a4k
//
//  Same as DirectX - D3DXQuaternionConjugate
//

void vmp_QuatConjugateGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    pqD->x = -pqA->x;
    pqD->y = -pqA->y;
    pqD->z = -pqA->z;
    pqD->w =  pqA->w;
}



//
//  Quaternion Negate
//

void vmp_QuatNegGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    vmp_QuatNormalizeGeneric(pqD, pqA);

    pqD->x = -pqD->x;
    pqD->y = -pqD->y;
    pqD->z = -pqD->z;
}



//
//  (Unit) Quaternion Inverse D=A^-1
//
//  Same as DirectX - D3DXQuaternionInverse
//
//  Unit Quaternion

void vmp_QuatInvGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    float fLen;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

        //  Sum of squares
    fLen = pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z + pqA->w * pqA->w;

    if (0.0f != fLen)                   // NOT absolute zero?
      {
        fLen = 1.0f / fLen;             // 1/r
        pqD->x = -pqA->x * fLen;
        pqD->y = -pqA->y * fLen;
        pqD->z = -pqA->z * fLen;
        pqD->w =  pqA->w * fLen;
      }
    else                                // (INVALID {0,0,0,0} not a quaternion)
      {
        pqD->x = pqD->y = pqD->z = 0.0f;
        pqD->w = 1.0f;
      }
}



//
//  Quaternion rotation from Euler (Yaw Pitch Roll) Angles
//      Yaw:Y-axis  Pitch:X-axis  Roll:Z-axis
//
//  Same as DirectX - D3DXQuaternionRotationYawPitchRoll (with DIRECTX_EQUIV)

void vmp_QuatRotYawPitchRollGeneric( vmpQuat * const pqD, float fYaw, float fPitch, float fRoll )
{
    float fYawSin, fPitchSin, fRollSin;
    float fYawCos, fPitchCos, fRollCos;
    float fPYCos, fPYSin, fPCYS, fPSYC;

    vmp_FSinCosGeneric( &fYawSin,   &fYawCos,   fYaw   * 0.5f );    // Yaw/2
    vmp_FSinCosGeneric( &fPitchSin, &fPitchCos, fPitch * 0.5f );    // Pitch/2
    vmp_FSinCosGeneric( &fRollSin,  &fRollCos,  fRoll  * 0.5f );    // Roll/2

    fPYCos = fPitchCos * fYawCos;
    fPYSin = fPitchSin * fYawSin;
    fPCYS =  fPitchCos * fYawSin;
    fPSYC =  fPitchSin * fYawCos;

    pqD->x = (fRollCos * fPSYC)  + (fRollSin * fPCYS);
    pqD->w = (fRollCos * fPYCos) + (fRollSin * fPYSin);

#if 0 //ndef DIRECTX_EQUIV
    pqD->y = (fRollSin * fPYCos) - (fRollCos * fPYSin);
    pqD->z = (fRollCos * fPCYS)  - (fRollSin * fPSYC);
#else   // DIRECTX equivalent               // (left hand?)
    pqD->y = (fRollCos * fPCYS)  - (fRollSin * fPSYC);
    pqD->z = (fRollSin * fPYCos) - (fRollCos * fPYSin);
#endif
}



//
//  Quaternion Multiplication |D|=|A|x|B|
//
//  Same as DirectX - D3DXQuaternionMultiply
//              NOTE: DirectX |D|=|B|x|A|

void vmp_QuatMulGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB)
{
    vmpQuat quat;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    quat.x = pqA->x * pqB->w + pqA->y * pqB->z - pqA->z * pqB->y + pqA->w * pqB->x;
    quat.y = pqA->y * pqB->w - pqA->x * pqB->z + pqA->w * pqB->y + pqA->z * pqB->x;
    quat.z = pqA->z * pqB->w + pqA->w * pqB->z + pqA->x * pqB->y - pqA->y * pqB->x;
    pqD->w = pqA->w * pqB->w - pqA->z * pqB->z - pqA->y * pqB->y - pqA->x * pqB->x;
    pqD->x = quat.x;
    pqD->y = quat.y;
    pqD->z = quat.z;
}



//
//  Quaternion Square
//

void vmp_QuatSquareGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    float s;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    s = 2.0f * pqA->w;

    pqD->w = (pqA->w * pqA->w - pqA->x * pqA->x - pqA->y * pqA->y - pqA->z * pqA->z);
    pqD->x = s * pqA->x;
    pqD->y = s * pqA->y;
    pqD->z = s * pqA->z;
}



//
//  Quaternion Division
//

void vmp_QuatDivGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB)
{
    vmpQuat qTmp, qB;
    float s;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

    // invert vector
    qB.x = -pqB->x;
    qB.y = -pqB->y;
    qB.z = -pqB->z;
    qB.w =  pqB->w;

    vmp_QuatMulGeneric(pqD, pqA, &qTmp);
    vmp_QuatMulGeneric(&qB, &qB, &qB);  // square elements

    if (0.0f != qB.w)       // Not too close to zero?
      {
        s = 1.0f / qB.w;

        pqD->x = qTmp.x * s;
        pqD->y = qTmp.y * s;
        pqD->z = qTmp.z * s;
        pqD->w = qTmp.w * s;
      }
    else
      {
        pqD->x = qTmp.x;
        pqD->y = qTmp.y;
        pqD->z = qTmp.z;
        pqD->w = qTmp.w;
      }
}



//
//  Quaternion SquareRoot
//

void vmp_QuatSqrtGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    vmpQuat qT;
    float fx, fy, fMag, fLen;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    fLen = sqrtf(pqA->x * pqA->x + pqA->y * pqA->y + pqA->w * pqA->w);

    if (0.0f != fLen)                       // Not too close to zero?
      {
        fLen = 1.0f / fLen; 
      }
    else
      {
        fLen = 1.0f;
      }

    qT.x = pqA->x * fLen;
    qT.y = pqA->z * fLen;
    qT.z = pqA->z;
    qT.w = pqA->w * fLen;

    fMag = 1.0f / sqrtf( qT.w * qT.w + qT.x * qT.x );
    fx   =        sqrtf( (1.0f - qT.y) * 0.5f );
    fy   =        sqrtf( (1.0f + qT.y) * 0.5f );
    fLen =        sqrtf( fLen );

    pqD->x = qT.x * fLen * fx * fMag;
    pqD->y =        fLen * fy;
    pqD->z = qT.z;
    pqD->w = qT.w * fLen * fy * fMag;
}



//
//  (Pure) Quaternion Exponent
//
//  Same as DirectX - D3DXQuaternionExp (requires Normalized Quaternion)
//      (Pure quaternion to unit quaternion).  (Pure: w is ignored in calculation!)

void vmp_QuatExpGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    float fMag, fSin;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    fMag = sqrtf(pqA->x * pqA->x + pqA->y * pqA->y + pqA->z * pqA->z);

    if (0.0f != fMag)                       // Not too close to zero?
      {
        vmp_FSinCos(&fSin, &(pqD->w), fMag);

        fMag = fSin / fMag;

        pqD->x = pqA->x * fMag;
        pqD->y = pqA->y * fMag;
        pqD->z = pqA->z * fMag;
      }
    else
      {
        pqD->w = 1.0f;              // 1.0 = cos(0)
        pqD->x = pqD->y = pqD->z = 0.0f;
      }

}



//
//  (Unit) Quaternion Natural Log
//
//  Same as DirectX - D3DXQuaternionLn
//
//  Unit Quaternion

void vmp_QuatLnGeneric(vmpQuat * const pqD, const vmpQuat * const pqA)
{
    float fMag, fSin;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);

    fMag = acosf( pqA->w );
    fSin = sinf( fMag );

    pqD->w = 0.0f;

    if (0.0f != fSin)
      {
        fSin = fMag / fSin;

        pqD->x = pqA->x * fSin;
        pqD->y = pqA->y * fSin;
        pqD->z = pqA->z * fSin;
      }
    else
      {
        pqD->x = pqD->y = pqD->z = 0.0f;
      }
}



//
//  Normalized Quaternion to Rotation Matrix
//
//  Same as DirectX - D3DXMatrixRotationQuaternion

void vmp_QuatToMatrixGeneric(vmp3DMatrix dMx, const vmpQuat * const pqA)
{
    float fMag;
    float fxx, fyy, fzz, fwx, fwy, fwz, fxy, fxz, fyz;

    ASSERT_PTR4(dMx);
    ASSERT_PTR4(pqA);

    fxx = pqA->x * pqA->x;  // xx
    fyy = pqA->y * pqA->y;  // yy
    fzz = pqA->z * pqA->z;  // zz

    fMag = fxx + fyy + fzz + (pqA->w * pqA->w);
    if (0.0f < fMag)
      {
        fMag = 2.0f / fMag;

        fxx *= fMag;
        fyy *= fMag;
        fzz *= fMag;
      }
    
    fwx = pqA->w * pqA->x * fMag;   // wx
    fwy = pqA->w * pqA->y * fMag;   // wy
    fwz = pqA->w * pqA->z * fMag;   // wz

    fxy = pqA->x * pqA->y * fMag;   // xy
    fxz = pqA->x * pqA->z * fMag;   // xz
    fyz = pqA->y * pqA->z * fMag;   // yz

    dMx[0][0] = 1.0f - (fyy + fzz);
    dMx[0][1] =        (fxy + fwz);
    dMx[0][2] =        (fxz - fwy);
    dMx[0][3] = 0.0f;

    dMx[1][0] =        (fxy - fwz);
    dMx[1][1] = 1.0f - (fxx + fzz);
    dMx[1][2] =        (fyz + fwx);
    dMx[1][3] = 0.0f;

    dMx[2][0] =        (fxz + fwy);
    dMx[2][1] =        (fyz - fwx);
    dMx[2][2] = 1.0f - (fxx + fyy);
    dMx[2][3] = 0.0f;

    dMx[3][0] = 0.0f;
    dMx[3][1] = 0.0f;
    dMx[3][2] = 0.0f;
    dMx[3][3] = 1.0f;
}



//
//  Rotation Matrix to Quaternion
//
//  Same as DirectX - D3DXQuaternionRotationMatrix

void vmp_QuatMatrixGeneric(vmpQuat * const pqD, const vmp3DMatrix aMx )
{
    float fSum, *pf, r, s;
    int i,j,k;

    ASSERT_PTR4(aMx);
    ASSERT_PTR4(pqD);

    fSum = aMx[0][0] + aMx[1][1] + aMx[2][2] + aMx[3][3];

        // check the diagonal

    if (fSum >= 1.0f)                   // Positive Diagonal
      {
        s = sqrt(fSum) * 0.5f;

        pqD->w = s;
        pqD->x = (aMx[1][2] - aMx[2][1]) * s;
        pqD->y = (aMx[2][0] - aMx[0][2]) * s;
        pqD->z = (aMx[0][1] - aMx[1][0]) * s;
      }
    else                            // Negative Diagonal
      {                             // Order the diagonal elements
        pf = (float *)pqD;          // Float elements base {XYZ}W


        if (aMx[0][0] > aMx[1][1])
          {
            k = 0x9;            // 1001 00b  2 1 0   k j i
            i = 0;
          }
        else
          {
            k = 0x2;            // 0010 01b  0 2 1   k j i
            i = 1;
          }

        if (aMx[2][2] > aMx[i][i])
          {
            k = 0x4;            // 0100 10b  1 0 2   k j i
            i = 2;
          }

            //  {i,j,k} (n % 3) thus ; {0,1,2} {1,2,0} {2,0,1}    
        j = k & 3;
        k = (k >> 2);

            //  'i' is index to biggest element

        r = sqrtf(aMx[i][i] - aMx[j][j] - aMx[k][k] + 1.0f);

        *(pf+i) = r * 0.5f;

        if (0.0f != r)
          {
            s = 1.0f / (2.0f * r);
          }
        else
          {
            s = 1.0f;
          }

        *(pf+j) = (aMx[i][j] + aMx[j][i]) * s;
        *(pf+k) = (aMx[i][k] + aMx[k][i]) * s;
        pqD->w =  (aMx[j][k] - aMx[k][j]) * s;
      }
}



//
//  Interpolate between two Quaternions using Spherical linear interpolation
//    from pqA to pqB distance fDist
//
//  Same as DirectX - D3DXQuaternionSlerp

void vmp_QuatSlerpGeneric(vmpQuat * const pqD, const vmpQuat * const pqA,
        const vmpQuat * const pqB, float fDist)
{
    float fCos, fSin, fAngle, fS0, fS1;
    vmpQuat vT;

    ASSERT_PTR4(pqD);
    ASSERT_PTR4(pqA);
    ASSERT_PTR4(pqB);

        //  Calculate Dot Product (cosine of angle between quaternions)!

    fCos = pqA->x * pqB->x + pqA->y * pqB->y + pqA->z * pqB->z + pqA->w * pqB->w;

    if (0.0f > fCos)                                // Quaterions angle Obtuse? (-)?
      {
        fCos = -fCos;                               // Yes, then flip B! (force acute) (+)
        vT.x = -pqB->x;
        vT.y = -pqB->y;
        vT.z = -pqB->z;
        vT.w = -pqB->w;
      }
    else                                            // acute angle (<=90 degrees)
      {
        vT.x =  pqB->x;
        vT.y =  pqB->y;
        vT.z =  pqB->z;
        vT.w =  pqB->w;
      }

        // Calculate coefficients

    fS0 = 1.0f - fDist;

    if (SINGLE_PRECISION < (1.0 - fCos))                // Far enough apart
      {
        fAngle = acosf(fCos);
        fSin = 1.0f / sinf(fAngle);

        fS0 = sinf(fS0   * fAngle) * fSin;
        fS1 = sinf(fDist * fAngle) * fSin;
      }
    else                                                // Too close together
      {        
        fS1 = fDist;
      }

        // Final scaled quaternion

    pqD->x = fS0 * pqA->x + fS1 * vT.x;
    pqD->y = fS0 * pqA->y + fS1 * vT.y;
    pqD->z = fS0 * pqA->z + fS1 * vT.z;
    pqD->w = fS0 * pqA->w + fS1 * vT.w;
}










