  
//
//  Quat3D.h - Quaternions Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to accomodate the book.
//
//      This module is designed for fast 3D math
//
//  Created - 30 Apr 2002 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Kari Games Master Definitions
#include    "vmp3D.h"



#ifndef __Quat3D_H
#define __Quat3D_H

#ifdef __cplusplus
extern "C" {
#endif


typedef void (*vmp_QuatCopyProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatCopyProc vmp_QuatCopy;

typedef void (*vmp_QuatConjugateProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatConjugateProc vmp_QuatConjugate;

typedef void (*vmp_QuatInvProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatInvProc vmp_QuatInv;

typedef void (*vmp_QuatSquareProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatSquareProc vmp_QuatSquare;

typedef void (*vmp_QuatSqrtProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatSqrtProc vmp_QuatSqrt;

typedef void (*vmp_QuatDotProductProc)(float * const pfD, const vmpQuat * const pqA, const vmpQuat * const pqB);
extern vmp_QuatDotProductProc vmp_QuatDotProduct;
extern vmp_QuatDotProductProc vmp_QuatDotProductFast;

typedef void (*vmp_QuatMagnitudeProc)(float * const pfD, const vmpQuat * const pqA);
extern vmp_QuatMagnitudeProc vmp_QuatMagnitude;
extern vmp_QuatMagnitudeProc vmp_QuatMagnitudeFast;

typedef void (*vmp_QuatNormalizeProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatNormalizeProc vmp_QuatNormalize;
extern vmp_QuatNormalizeProc vmp_QuatNormalizeFast;

typedef void (*vmp_QuatAddProc)(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
extern vmp_QuatAddProc vmp_QuatAdd;

typedef void (*vmp_QuatSubProc)(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
extern vmp_QuatSubProc vmp_QuatSub;

typedef void (*vmp_QuatNegProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatNegProc vmp_QuatNeg;

typedef void (*vmp_QuatRotAxisProc)( vmpQuat * const pqD, const vmp3DVector * const pvAxis, float fAngle );
extern vmp_QuatRotAxisProc vmp_QuatRotAxis;

typedef void (*vmp_QuatToRotAxisProc)( vmp3DVector * const pvDAxis, float * const pfAngle, const vmpQuat * const pqAQuat );
extern vmp_QuatToRotAxisProc vmp_QuatToRotAxis;

typedef void (*vmp_QuatRotYawPitchRollProc)( vmpQuat * const pqD, float fYaw, float fPitch, float fRoll );
extern vmp_QuatRotYawPitchRollProc vmp_QuatRotYawPitchRoll;

typedef void (*vmp_QuatMulProc)(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
extern vmp_QuatMulProc vmp_QuatMul;

typedef void (*vmp_QuatDivProc)(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
extern vmp_QuatDivProc vmp_QuatDiv;

typedef void (*vmp_QuatExpProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatExpProc vmp_QuatExp;

typedef void (*vmp_QuatLnProc)(vmpQuat * const pqD, const vmpQuat * const pqA);
extern vmp_QuatLnProc vmp_QuatLn;

typedef void (*vmp_QuatToMatrixProc)(vmp3DMatrix dMx, const vmpQuat * const pqA);
extern vmp_QuatToMatrixProc vmp_QuatToMatrix;

typedef void (*vmp_QuatMatrixProc)(vmpQuat * const pqD, const vmp3DMatrix aMx );
extern vmp_QuatMatrixProc vmp_QuatMatrix;

typedef void (*vmp_QuatSlerpProc)(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB, float fDist);
extern vmp_QuatSlerpProc vmp_QuatSlerp;



#ifdef __cplusplus
}
#endif

#endif  //__Quat3D_H


