  
//
//  Quat3DEx.h - Quaternions Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to accomodate the book.
//
//      This module is designed for fast 3D math
//
//  Created - 30 Apr 2002 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Kari Games Master Definitions
#include    "CpuAsm.h"
#include    "vmp3D.h"


#ifndef __Quat3DEx_H
#define __Quat3DEx_H

#ifdef __cplusplus
extern "C" {
#endif


void Quat3DSetup( const CpuInfo * const pcinfo );



void vmp_QuatCopyGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatConjugateGeneric(  vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatConjugateAsm3DNow( vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatConjugateAsmSSE(   vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatInvGeneric(  vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatInvAsm3DNow( vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatInvAsmSSE(   vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatSquareGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatSqrtGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatDotProductGeneric(  float * const pfD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatDotProductAsm3DNow( float * const pfD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatDotProductAsmSSE(   float * const pfD, const vmpQuat * const pqA, const vmpQuat * const pqB);

void vmp_QuatMagnitudeGeneric(  float * const pfD, const vmpQuat * const pqA);
void vmp_QuatMagnitudeAsm3DNow( float * const pfD, const vmpQuat * const pqA);
void vmp_QuatMagnitudeAsmSSE(   float * const pfD, const vmpQuat * const pqA);

void vmp_QuatMagnitudeFastAsm3DNow( float * const pfD, const vmpQuat * const pqA);
void vmp_QuatMagnitudeFastAsmSSE(   float * const pfD, const vmpQuat * const pqA);

void vmp_QuatNormalizeGeneric(  vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatNormalizeAsm3DNow( vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatNormalizeAsmSSE(   vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatNormalizeFastAsm3DNow( vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatNormalizeFastAsmSSE(   vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatAddGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatSubGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatNegGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatRotAxisGeneric( vmpQuat * const pqD, const vmp3DVector * const pvA, float fAngle );
void vmp_QuatToRotAxisGeneric( vmp3DVector * const pvD, float * const pfAngle, const vmpQuat * const pqA );

void vmp_QuatRotYawPitchRollGeneric( vmpQuat * const pqD, float fYaw, float fPitch, float fRoll );

void vmp_QuatMulGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatMulAsm3DNow(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);
void vmp_QuatMulAsmSSE(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);

void vmp_QuatDivGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB);

void vmp_QuatExpGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);
void vmp_QuatLnGeneric(vmpQuat * const pqD, const vmpQuat * const pqA);

void vmp_QuatToMatrixGeneric(vmp3DMatrix dMx, const vmpQuat * const pqA);
void vmp_QuatMatrixGeneric(vmpQuat * const pqD, const vmp3DMatrix aMx );

void vmp_QuatSlerpGeneric(vmpQuat * const pqD, const vmpQuat * const pqA, const vmpQuat * const pqB, float fDist);



#ifdef __cplusplus
}
#endif

#endif  //__Quat3DEx_H


