
//
//  Bench.cpp - ASCII to Floating-Point Conversion
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      This demonstrates ASCII to Float-Point conversions.
//
//  Created - 28 Mar, 2002 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Master Definitions

    //  3rd Party Files

#include    <windows.h>
#include    <iostream.h>
#include    <assert.h>
#include    <stdio.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU Module
#include    "ram.h"                                     // Memory Module
#include    "util.h"                                    // General purpose utilities
#include    "vmp3D.h"




//
//  Process the Float(s) file
//
void taskFloat( const char *pStr, uint nLen)
{
    uint n, nCnt;
    ulong tDelay;
    double fFast, fSlow;
    const char *p;
    char szNum[40];

    pStr = skipdelim(pStr);

    cout << "Comparing Slow atof() versus fast ASCIItoFlt() method to verify accuracy..." << endl;

    CpuDelaySet();

    p = pStr;
    for (nCnt = 0; *p; nCnt++)
      {
        p = getword(szNum, p);
        p = skipdelim(p);

        fSlow = atof( szNum );
        fFast = ASCIItoDouble( szNum );

        if (!vmp_IsDEqual(fFast, fSlow, DOUBLE_PRECISION))
          {
            cout << "Mismatch found " << fFast << " vs " << fSlow << endl;
            break;
          }
      }

    tDelay = CpuDelayCalc();

    cout << nCnt << " floats, with a conversion time of " << tDelay << " cycles.\n" << endl;

        //  ____________________________________________

    cout << "Running Slow FP conversion..." << endl;

    CpuDelaySet();

    p = pStr;
    while (*p)
      {
        p = getword(szNum, p);
        p = skipdelim(p);

        fSlow = atof( szNum );
      }

    tDelay = CpuDelayCalc();

    cout << "                       ...with a time of " << tDelay << " cycles.\n" << endl;

        //  ____________________________________________

    cout << "Running Fast FP conversion..." << endl;

    CpuDelaySet();

    p = pStr;
    while (*p)
      {
        p = getword(szNum, p);
        p = skipdelim(p);

        fSlow = ASCIItoDouble( szNum );
      }

    tDelay = CpuDelayCalc();

    cout << "                       ...with a time of " << tDelay << " cycles.\n" << endl;

        //  ____________________________________________

    cout << "Running Fast FP conversion w/Self advancing pointer..." << endl;

    CpuDelaySet();

    p = pStr;
    while (*p)
      {
        fSlow = pASCIItoDouble( &p );
        p = skipdelim(p);
      }

    tDelay = CpuDelayCalc();

    cout << "                       ...with a time of " << tDelay << " cycles.\n" << endl;

    cout << "____________________________________________________________________________\n";
    cout << "AND the following is why you use a small private buffer for atof()..." << endl;

    n = 0;
    p = pStr;
    while (*p)
      {
        fSlow = atof( p );          // and sscanf() take about the same amount of time!
        p = skipword(p);
        p = skipdelim(p);

        if (500==++n) break;
      }

    cout << "                       ...TOLD YA!!!\n" << endl;

    if(*p)
      {
        cout << " Note that instead of " << nCnt << " floats, I ended it early after only " << n << "!" << endl; 
        cout << "\n... this could have taken a few hours..." << endl;
        cout << "...Did you know sscanf() is just as bad? Just a little food for thought!\n" << endl;
      }
}



//
//  Parse the command line.
//

static bool ParseCmdLine( char *szFPath )
{
    bool ret, bHelp;
    char *b, *e, *szCmd;
    uint nLen;

    ASSERT_PTR(szFPath);

    *szFPath = '\0';                                    // Pre-terminate

    bHelp = ret = false;

    szCmd = GetCommandLine();                           // Full path & arguments

        //  If Leading Quote then find trailing quote

    if ( '\"' == *szCmd )
      {
        szCmd++;

        b = strchr(szCmd, '\"');
        if ( NULL == b )
          {
            return false;
          }

        b++;
      }
        //  Skip filename
    else
      {
        b = skipword( szCmd );
      }

    b = skipdelim( b );                                 // Skip leading space

        //  Loop for all arguments in command line
        //  NOTE: Could be file or folder!
    while (*b)
      {
        e = skipword( b );
        nLen = e-b;

        if ('-' == *b)
          {
            switch(tolower(*(b+1)))
              {
                case 'h':
                    bHelp = true;
                    break;

                default:
                    break;
              }
          }
        else
          {
            if (*b)
              {
                strcpy(szFPath, b);
                ret = true;                             // At least one item in list!
              }

          }

        b = skipdelim( e );
      }

    if ( bHelp )
      {
        cout << "\nAtoFlt {options} file.TXT\n\n";
        cout << "   -h   Help\n" << endl;
      }

    return ret;
}



int main(int argc, char *argv[])
{
    char szBuf[ CPU_SZBUF_MAX ];
    char *pStr, szFPath[ MAX_PATH ];
    CpuInfo cinfo;
    ulong nLen;
    int rval;

    CpuDetect( &cinfo );                                // Detect CPU
    
    cout << "\nASCII to Floating-Point speed demonstration.\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    ramSetup( &cinfo );                                 // Initialize Memory System

    rval = -1;
    pStr = NULL;

        //  Parse the command line

    if (!ParseCmdLine( szFPath ))   
      {
        cout << "Error parsing command line!" << endl;
      }

        //  Load ASCII Float data file into memory
        //  NOTE: Text file has an appended zero terminator!

    else if (!fileLoad( szFPath, (byte **)&pStr, &nLen ))
      {
        cout << "Couldn't find/load file!" << endl;
      }

    else if (0L==nLen)
      {
        cout << "Empty file!" << endl;
      }

    else
      {
        taskFloat(pStr, nLen);
        rval = 0;
      }

        //  Termination - Destruction

    if (NULL != pStr)
      {
        ramRelease((byte**)&pStr);
      }

    cout << "Fin\n\n" << flush;

    return rval;
}
