
//
//  Root.cpp - (Main / Root) Core
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      This is the memory, file, management, module header.
//
//  Win32 Console Application
//
//  Created - 13 Mar, 2002 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Master Definitions

    //  3rd Party Files

#include    <windows.h>
#include    <iostream.h>
#include    <assert.h>
#include    <stdio.h>
#include    <limits.h>

    //  Book Development Library Modules
    
#include    "CpuAsm.h"                                  // CPU Module
#include    "vmp3D.h"                                   // VMP utils
#include    "ram.h"                                     // Memory Module
#include    "util.h"                                    // General purpose utilities
#include    "Root.h"
#include    "ase.h"                                     // ASE file parser


//  Local Data
AppData app;




//
//  Parse the command line.
//

static bool ParseCmdLine( char *szFPath )
{
    bool ret, bHelp;
    char *b, *e, *szCmd;
    uint nLen;

    ASSERT_PTR4(szFPath);

    LOG( "ase2vmp - ASE to VMP Poly Converter.\n" );

    *szFPath = '\0';                                    // Pre-terminate

    bHelp = ret = false;

    szCmd = GetCommandLine();                           // Full path & arguments

        //  If Leading Quote then find trailing quote

    if ( '\"' == *szCmd )
      {
        szCmd++;

        b = strchr(szCmd, '\"');
        if ( NULL == b )
          {
            return false;
          }

        b++;
      }
        //  Skip filename
    else
      {
        b = skipword( szCmd );
      }

    b = skipspace( b );                                 // Skip leading space

        //  Loop for all arguments in command line
        //  NOTE: Could be file or folder!
    while (*b)
      {
        e = skipword( b );
        nLen = e-b;

        if ('-' == *b)
          {
            switch(tolower(*(b+1)))
              {
                case 'h':
                    bHelp = true;
                    break;

                case 'v':
                    app.bVerbose = true;
                    break;

                default:
                    break;
              }
          }
        else
          {
            if (*b)
              {
                strcpy(szFPath, b);
                ret = true;                             // At least one item in list!
              }

          }

        b = skipspace( e );
      }

    if ( bHelp )
      {
        LOG( "\nase2vmp {options} file.ASE\n\n" );
        LOG( "   -h   Help\n" );
        LOG( "   -v   Verbose printing  Default: off\n" );
        LOG( "\n" );
      }

    return ret;
}



int main(int argc, char *argv[])
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];
    char *pStr, szFPath[ MAX_PATH ];
    int rval;
    ulong nLen;
    Node    nodeHead;
    rdbInfo stat;

    CpuDetect( &cinfo );                                // Detect CPU
    
    cout << "\n(Quad) Vector Addition & Subtraction Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    ramSetup( &cinfo );                                 // Initialize Memory System

    rval = -1;
    pStr = NULL;

    ramZero(&app, sizeof(app));
    ramZero(&nodeHead, sizeof(nodeHead));
    ramZero(&stat, sizeof(stat));

floatXlat = 0;
        //  Parse the command line

    if (!ParseCmdLine( szFPath ))   
      {
        ELOG( "Error parsing command line!\n" );
      }

        //  Load ASE file into memory

    else if (!fileLoad( szFPath, (byte **)&pStr, &nLen ))
      {
        ELOG( "Couldn't find/load file!\n" );
      }

    else if (0L==nLen)
      {
        ELOG( "Empty file!\n" );
      }

    else if (!aseParse( pStr, nLen, &nodeHead, &stat ))
      {
        ELOG( "ASE Parsing Error!\n" );
      }

    else if (!aseRDB(&nodeHead, &stat))
      {
        ELOG( "ASE Database Manipulation Errror!\n" );
      }

    else if (!aseWriteTextureC( szFPath, &nodeHead ))
      {
        ELOG( "ASE Export of Texture failure!\n" );
      }

    else
      {
        ELOG( "SUCCESSFUL!\n" );
        rval = 0;
      }


        //  Termination - Destruction

    if (NULL != pStr)
      {
        ramRelease((byte**)&pStr);
      }

    cout << "\nThere were " << floatXlat << " floats converted from ASCII to double!" << endl;
    cout << "Fin\n\n" << flush;

    return rval;
}
