
//
//  CpuAsm.h - Cpu Detection - "The Death and Rebirth of the X86 Assembly Language Programmer"
//
//  CpuX86.asm - 80(X)86 Cpu definitions
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to accomodate the book.
//
//  Created - 27 Jun 98 - James Leiterman
//
//  Last Modification
//
//      23 Jul 2001 - JCL - Brought up to date.
//
//  stab=4
//


#include    "KariType.h"                                // Kari Games Master Definitions



#ifndef __CpuAsm_H
#define __CpuAsm_H

#ifdef __cplusplus
extern "C" {
#endif



//
//  CPU definition Id's
//

typedef enum
{
    CPU_UNKNOWN     =   0,
    CPU_386         =   3,                              // 80386
    CPU_486         =   4,                              // 80486
    CPU_PENTIUM     =   5,                              // P5 (Pentium)
    CPU_PII         =   6,                              // Pentium Pro / PII / PIII / PIV
} CPU;



//
//  CPU bit flags
//

typedef enum
{
    CPUBITS_FPU         =   0x00000001,                 // FPU flag
    CPUBITS_MMX         =   0x00000002,                 // MMX flag
    CPUBITS_3DNOW       =   0x00000004,                 // 3DNow flag
    CPUBITS_FXSR        =   0x00000008,                 // Fast FP Store
    CPUBITS_SSE         =   0x00000010,                 // Katmai NI
    CPUBITS_SSE2        =   0x00000020,
    CPUBITS_3DNOW_MMX   =   0x00000040,                 // 3DNow (MMX Extensions)
    CPUBITS_3DNOW_EXT   =   0x00000080,                 // 3DNow (Extensions)
    CPUBITS_3DNOW_SSE   =   0x00000100,                 // 3DNow! Professional
} CPUBITS;



//
//  CPU Vendors
//

typedef enum
{
    CPUVEN_UNKNOWN      =   0,                          // Unknown
    CPUVEN_INTEL        =   1,                          // Intel
    CPUVEN_AMD          =   2,                          // AMD (Advanced Micro Devices)
    CPUVEN_CYRIX        =   3,                          // Cyrix
    CPUVEN_CENTAUR      =   4,                          // IDT Centaur (WinChip)
    CPUVEN_NATIONAL     =   5,                          // National Semiconductor
    CPUVEN_UMC          =   6,                          // UMC
    CPUVEN_NEXGEN       =   7,                          // NexGen
    CPUVEN_RISE         =   8,                          // Rise
    CPUVEN_TRANSMETA    =   9,                          // Transmeta
} CPUVEN;


//
//  Cpu Information
//

typedef struct CpuInfoType
{
    uint    nCpuId;                                     // Cpu type Identifier
    uint    nFpuId;                                     // Floating Point Unit Identifier
    uint    nBits;                                      // Feature Bits
    uint    nMfg;                                       // Manufacturer
} CpuInfo;




void vmp_SIMDEntryGeneric(void);
void vmp_SIMDEntryAsmMMX(    void);
void vmp_SIMDEntryAsm3DNow(  void);

void vmp_SIMDExitGeneric(void);
void vmp_SIMDExitAsmMMX(    void);
void vmp_SIMDExitAsm3DNow(  void);



//
//  SIMD Entry Point
//
#ifdef CC_VMP_WIN32
typedef void (*vmp_SIMDEntryProc)(void);                // Function pointer
extern vmp_SIMDEntryProc vmp_SIMDEntry;
#else
#define vmp_SIMDEntry()
#endif



//
//  SIMD Exit Point
//
#ifdef CC_VMP_WIN32
typedef void (*vmp_SIMDExitProc)(void);                 // Function pointer
extern vmp_SIMDExitProc vmp_SIMDExit;
#else
#define vmp_SIMDExit()
#endif


    //
    //  Function Prototypes
    //

void CpuDetect( CpuInfo * const pInfo );

#define CPU_SZBUF_MAX   110
char *CpuInfoStr( char *pszBuf, const CpuInfo * const pInfo );


void CpuDelaySet( void );
ulong CpuDelayCalc( void );



#ifdef __cplusplus
}
#endif

#endif  //__CpuAsm_H

